local addonInfo, privateVars = ...

function nkRebuff.Analyzer()

	local name = "nkRebuff.Analyzer"

	local analyzer = EnKai.uiCreateFrame("nkWindowElement", name, privateVars.uiElements.context)

	local grid = EnKai.uiCreateFrame("nkGrid", name .. ".grid", analyzer:GetContent())
	local slider = EnKai.uiCreateFrame("nkScrollbox", name .. ".slider", analyzer:GetContent())	
	local analyzeHint = EnKai.uiCreateFrame("nkInfoText", "nkRB.analyzer.hint", analyzer:GetContent())	
	local button = EnKai.uiCreateFrame("nkButton", name .. ".button", analyzer:GetContent())
		
	analyzer:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 200, 40)
	--analyzer:SetBackgroundColor(0, 0, 0, .5)
	analyzer:SetWidth(548)
	analyzer:SetHeight(390)
	
	analyzer:SetTitle("nkRebuff Analyzer")	
	
	grid:SetHeaderHeight(25)
	grid:SetPoint("TOPLEFT", analyzer:GetContent(), "TOPLEFT", 10, 10)
	grid:SetHeaderLabelColor(0.925, 0.894, 0.741, 1)
	grid:SetBorderColor(0.8, 0.8, 0.8, 1)
	grid:SetBodyColor(0.133, 0.133, 0.133, 1)
	--grid:SetBodyHighlightColor(0, 0, 0, 1)
	--grid:SetLabelHighlightColor(1, 1, 1, 1)
	grid:SetHeaderFontSize(11)
	grid:SetFontSize(11)
	--grid:SetTransparentHeader()
	
	local cols = {	{ align = 'center', texture = true, textureType = 'Rift', width = 30}, 
					{ header = privateVars.langTexts.headerDescription, width = 200 }, 
					{ align = 'center', header = privateVars.langTexts.abilityType, width = 75 },
					{ align = 'center', header = privateVars.langTexts.duration, width = 75 },
					{ align = 'center', header = privateVars.langTexts.castBy, width = 75 },
					{ align = 'center', texture = true, textureType = 'nkRebuff', textureWidth = 14, textureHeight = 14, header = privateVars.langTexts.storeThis, width = 80 }}
	
	grid:SetVisible(false)	
	grid:Layout (cols, 15)
	
	slider:SetPoint("TOPLEFT", grid, "TOPRIGHT", 0, 18)
	slider:SetHeight(grid:GetHeight() - 18)
	slider:SetRange(1, 1)
	slider:SetVisible(false)
	
	analyzeHint:SetPoint("TOPLEFT", grid, "BOTTOMLEFT", 0, 5)
	analyzeHint:SetWidth(grid:GetWidth())
	analyzeHint:SetText(privateVars.langTexts.analyzeHint)
	analyzeHint:SetFontSize(10)
	analyzeHint:SetHeight(50)
	analyzeHint:SetFontColor(0.925, 0.894, 0.741, 1)
	analyzeHint:SetType('info')
	analyzeHint:SetIconAlign('top')	
	
	button:SetPoint ("BOTTOMCENTER", analyzer:GetContent(), "BOTTOMCENTER", 0, -10)
	button:SetText (privateVars.langTexts.storeSelected)
	button:SetScale (0.8)
	
	local dataPlayer = {}
	local dataOther = {}
	local gridValues = {}
	
	function analyzer:GridUpdate()				
	
		--print ('-- grid update --')
	
		gridValues = {}
	
		analyzer:ProcessData(dataPlayer)
		analyzer:ProcessData(dataOther)
		
		grid:SetCellValues(gridValues)		
		analyzer:CheckSlider()
		
	end
	
	function analyzer:ProcessData (data)
		
		for id, v in pairs (data) do
			local useIcon, itemType, color, duration = false, "buff", nil, ""
			
			if v.use == true then useIcon = "gfx/statusGreen.png" else useIcon = "gfx/statusRed.png" end
			
			if v.debuff == true then 
				itemType = privateVars.langTexts.debuff
				color = {1, 0, 0, 1}
			elseif v.curse == true then 
				itemType = privateVars.langTexts.curse
				color = {0.839, 0.498, 1, 1}
			elseif v.cooldown == true then
				itemType = privateVars.langTexts.cooldown
				color = {0, 0.580, 1, 1}
			elseif v.isPet == true then
				itemType = privateVars.langTexts.buffOnPet
				color = {.7, .7, .7, 1}
			else
				itemType = privateVars.langTexts.buff
			end
			
			if v.duration ~= nil then duration = string.format(privateVars.langTexts.durationValue, v.duration) end
						
			table.insert (gridValues, {{ key = id, value = v.icon}, {value = v.name, color = color}, { value = itemType, color = color }, { value = duration, color = color }, { value = v.caster, color = color }, {value = useIcon, color = color} })
		end
		
	end
	
	function analyzer:CheckSlider ()
		if (#gridValues-15) > 0 then
			slider:SetRange (1, (#gridValues-15))
			slider:SetVisible(true)
			analyzer:SetWidth(562)
		else
			slider:SetVisible(false)
			analyzer:SetWidth(548)
		end
	end
	
	function analyzer:BuffAdd (unit, buffs, isPet)

		--print ('analyzer - buffAdd')
	
		if unit ~= nil then
			local buffDetails
			local unitDetails = Inspect.Unit.Detail (unit)
			
			local playerDetails = nkRebuff.getUnitDetails ('player')
			local petDetails = nkRebuff.getUnitDetails ('player.pet')	
			local playerTarget = Inspect.Unit.Detail ('player.target')
			
			local isPet = false
			
			if playerTarget ~= nil and unitDetails.name == playerTarget.name then -- Vom Spieler auf Ziel gewirkt
				buffDetails = Inspect.Buff.Detail('player.target', buffs)
			elseif petDetails ~= nil and unitDetails.name == petDetails.name then -- auf Pet gewirkt
				buffDetails = Inspect.Buff.Detail('player.pet', buffs)
				isPet = true
			elseif unitDetails.name == playerDetails.name then -- andere Wirkungen
				buffDetails = Inspect.Buff.Detail('player', buffs)			
			end
			
			analyzer:ProcessBuff(buffDetails, playerDetails, petDetails, isPet, unit)
		end
	end
	
	function analyzer:ProcessBuff (buffDetails, playerDetails, petDetails, isPet, unit)

		if buffDetails == nil or playerDetails == nil then return end
				
		local hasNew = false
		
		local casterDetails = {}
	
		for id, buff in pairs (buffDetails) do
			--if buff.remaining ~= nil then -- filter buffs without a duration
			
				local buffKey = buff.type or buff.name
				if buffKey ~= nil then
			
					local caster = buff.caster
					
					if casterDetails[caster] == nil then casterDetails[caster] = Inspect.Unit.Detail(caster) end
										
					if casterDetails[caster] ~= nil then
						if casterDetails[caster].name == playerDetails.name then
						
							-- Vom Spieler gewirkt

							local addBuff = false
							if buff.debuff == true or buff.curse == true then -- debuff or curse
								if nkRBData[nkRBConfig.activeSet].debuffsTarget[buffKey] == nil then addBuff = true end
							else -- buff
								if nkRBData[nkRBConfig.activeSet].buffsPlayer[buffKey] == nil then addBuff = true end
							end
						
							if addBuff == true and dataPlayer[buffKey] == nil then
								dataPlayer[buffKey] = { id = id, unit = unit, name = buff.name, isPet = isPet, debuff = buff.debuff, curse = buff.curse, cooldown = false, duration = buff.duration, caster = privateVars.langTexts.player, ability = buff.abilityNew, icon = buff.icon, use = false }								
								hasNew = true
							end
						else
						
							-- Von NPC oder anderem Spieler gewirkt
						
							local addBuff = false
							if buff.debuff == false and buff.curse == false then
								if nkRBData[nkRBConfig.activeSet].buffsPlayer[buffKey] == nil then addBuff = true end
							else
								if petDetails ~= nil and petDetails.name == casterDetails[caster].name then -- nur debuffs vom pet sind interessant					
									if nkRBData[nkRBConfig.activeSet].buffsPlayer[buffKey] == nil then addBuff = true end
								end
							end
						
							if addBuff == true then 
								if petDetails ~= nil and petDetails.name == casterDetails[caster].name then					
									caster = privateVars.langTexts.pet
								elseif casterDetails[caster].calling ~= nil then
									caster = casterDetails[caster].calling
								else
									caster = privateVars.langTexts.npc
								end
								
								if dataOther[buffKey] == nil then
									dataOther[buffKey] = { id = id, unit = unit, name = buff.name, debuff = buff.debuff, curse = buff.curse, cooldown = false, duration = buff.duration, caster = caster, ability = buff.abilityNew, icon = buff.icon, use = false }									
									hasNew = true
								end
							end
						end
					end
				end
			--end
		end
				
		if hasNew == true then analyzer:GridUpdate() end
		
	end

	function analyzer:CooldownBegin(cooldowns)
	
		local hasNew = false
	
		for ability, cd in pairs (cooldowns) do
			if cd > 1.5 then			
			
				local abilityInfo = Inspect.Ability.New.Detail(ability)
				if abilityInfo ~= nil then 
					
					if nkRBData[nkRBConfig.activeSet].cooldownsPlayer[abilityInfo.idNew] == nil and dataPlayer[abilityInfo.idNew] == nil then
						dataPlayer[abilityInfo.idNew] = { name = string.gsub (abilityInfo.name, '\n', ' '), debuff = false, curse = false, cooldown = true, caster = privateVars.langTexts.player, ability = abilityInfo.idNew, icon = abilityInfo.icon, use = false }
						hasNew = true
					end
				end
			end
		end
		
		if hasNew == true then analyzer:GridUpdate() end
		
	end
	
	function analyzer:CreateIcon (buffType, key, xpos)
	
		local iconData = EnKai.tools.table.copy (nkRBConfig.iconDefaults)
		iconData.texturePath = nkRBData[nkRBConfig.activeSet][buffType][key].icon
		iconData.label = nkRBData[nkRBConfig.activeSet][buffType][key].name
		iconData.xpos = xpos
			
		nkRBIcons[nkRBConfig.activeSet][buffType][key] = EnKai.tools.table.copy(iconData)

		nkRBSets[nkRBConfig.activeSet][buffType][key] = {}
		nkRBSets[nkRBConfig.activeSet][buffType][key] = { active = true }
		
		if privateVars.uiElements.icons[buffType][key] == nil then 
			privateVars.uiElements.icons[buffType][key] = nkRebuff.createIcon (buffType, key, iconData) 
			
			if privateVars.uiElements.configUI ~= nil and privateVars.uiElements.configUI:GetVisible() == true then
				privateVars.uiElements.icons[buffType][key]:SetDragable(true)
			end			
		end	
		
	end
	
	function analyzer:StoreSelected (createFlag)
	
		local xpos = nkRBConfig.iconDefaults.xpos
	
		for k, v in pairs (dataPlayer) do
			if v.use == true then

				if v.debuff == true or v.curse == true then
					nkRBData[nkRBConfig.activeSet].debuffsTarget[k] = { name = v.name, ability = v.ability, icon = v.icon, duration = v.duration, hidden = false }
					if createFlag == true then 
						analyzer:CreateIcon ('debuffsTarget', k, xpos)
						xpos = xpos + (50 * (nkRBConfig.iconDefaults.scale / 100))					
					end
				elseif v.cooldown == true then
					nkRBData[nkRBConfig.activeSet].cooldownsPlayer[k] = { name = v.name, ability = v.ability, icon = v.icon, duration = 0, hidden = false }
					if createFlag == true then 
						analyzer:CreateIcon ('cooldownsPlayer', k, xpos)
						xpos = xpos + (50 * (nkRBConfig.iconDefaults.scale / 100))					
					end
				elseif v.isPet == true then
					nkRBData[nkRBConfig.activeSet].buffsPet[k] = { name = v.name, caster = v.caster, ability = v.ability, icon = v.icon, duration = v.duration, hidden = false }
					if createFlag == true then 
						analyzer:CreateIcon ('buffsPet', k, xpos)
						xpos = xpos + (50 * (nkRBConfig.iconDefaults.scale / 100))					
					end
				else
					nkRBData[nkRBConfig.activeSet].buffsPlayer[k] = { name = v.name, caster = v.caster, ability = v.ability, icon = v.icon, duration = v.duration, hidden = false }
					if createFlag == true then 
						analyzer:CreateIcon ('buffsPlayer', k, xpos)
						xpos = xpos + (50 * (nkRBConfig.iconDefaults.scale / 100))					
					end
				end
			end
		end
		
		for k, v in pairs (dataOther) do
			if v.use == true then
				if v.debuff == true or v.curse == true then
					-- nicht von interesse
				elseif v.cooldown == true then					
					-- nicht von interesse
				else
					nkRBData[nkRBConfig.activeSet].buffsPlayer[k] = { caster = v.caster, ability = v.ability, icon = v.icon, duration = v.duration, hidden = false }
					if createFlag == true then 
						analyzer:CreateIcon ('buffsPlayer', k, xpos)
						xpos = xpos + (50 * (nkRBConfig.iconDefaults.scale / 100))					
					end
				end
			end
		end
	
		dataPlayer = {}
		dataOther = {}		
	end
	
	function analyzer:ClearGrid()
	
		dataPlayer = {}
		dataOther = {}
		local buffs = Inspect.Buff.List('player')
		analyzer:BuffAdd('player', buffs)
		
		local buffs = Inspect.Buff.List('player.pet')
		if buffs ~= nil then analyzer:BuffAdd('player.pet', buffs) end
		analyzer:CheckSlider()
	end
	
	function analyzer:Toggle(clickedRow)
		if gridValues[clickedRow] == nil then return end
		
		local useIcon = "gfx/statusRed.png"
		
		if gridValues[clickedRow].use == true then 
			gridValues[clickedRow].use = false
		else
			gridValues[clickedRow].use = true
			useIcon = "gfx/statusGreen.png"
		end
			
		local key = grid:GetKey(clickedRow)
		
		for k, v in pairs (dataPlayer) do
			if k == key then
				dataPlayer[k].use = gridValues[clickedRow].use
				break
			end
		end
		
		for k, v in pairs (dataOther) do
			if k == key then
				dataOther[k].use = gridValues[clickedRow].use
				break
			end
		end
			
		grid:SetCellValue(clickedRow, 6, useIcon)	
	end
	
	function analyzer:CloseDialog(storeFlag, createFlag)
		if storeFlag == true then analyzer:StoreSelected(createFlag) end
		if privateVars.uiElements.configUI ~= nil then privateVars.uiElements.configUI:UpdateTrackGrid() end
		analyzer:SetVisible(false)			
	end
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].GridFinished, function ()
		local buffs = Inspect.Buff.List('player')
		analyzer:BuffAdd('player', buffs)
		
		local buffs = Inspect.Buff.List('player.pet')
		if buffs ~= nil then analyzer:BuffAdd('player.pet', buffs) end
		
		--analyzer:CheckSlider()
		grid:SetVisible(true)		
	end, name .. ".grid.GridFinished")	
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].MouseOver, function (_, row)
		local key = grid:GetKey(row + grid:GetRowPos() -1 )
		if dataPlayer[key] ~= nil then
			if dataPlayer[key].cooldown == true then
			 	pcall (Command.Tooltip, dataPlayer[key].ability)
			else
				pcall (Command.Tooltip, dataPlayer[key].unit, dataPlayer[key].id)
			end
		elseif dataOther[key] ~= nil then
			pcall(Command.Tooltip, dataOther[key].unit, dataOther[key].id)
		end
	end, name .. ".grid.MouseOver")
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].MouseOut, function (_, row)				
			Command.Tooltip(nil)		
	end, name .. ".grid.MouseOut")
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].LeftClick, function (_, clickedRow)
		analyzer:Toggle(clickedRow)
	end, name .. ".grid.LeftClick")
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].RightClick, function (_, clickedRow)				
		analyzer:Toggle(clickedRow)				
	end, name .. ".grid.RightClick")
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].WheelForward, function (_, newRow)
		slider:AdjustValue(newRow)
	end, name .. ".grid.WheelForward")
	
	Command.Event.Attach(EnKai.events[name .. ".grid"].WheelBack, function (_, newRow)
		slider:AdjustValue(newRow)
	end, name .. ".grid.WheelBack")
	
	Command.Event.Attach(EnKai.events[name .. ".slider"].ScrollboxChanged, function (_, newValue)
		grid:SetRowPos(math.floor(newValue), true)
	end, '[name].slider.ScrollboxChanged')
	
	Command.Event.Attach(EnKai.events[name..".button"].Clicked, function ()

		local hasSelected = false

		for k, v in pairs (dataPlayer) do
			if v.use == true then
				hasSelected = true
				break
			end
		end
		
		if hasSelected == false then		
			for k, v in pairs (dataOther) do
				if v.use == true then
					hasSelected = true
					break
				end
			end
		end
		
		if hasSelected == false then
			analyzer:CloseDialog(false, false)
		else
		
			local yesFunc = function ()
				analyzer:CloseDialog(true, true)
			end
			
			local noFunc = function ()
				analyzer:CloseDialog(true, false)
			end

			nkRebuff.confirmDialog (privateVars.langTexts.messages.createIcons, yesFunc, noFunc)
		end
		
	end, name .. ".button.Clicked")	
	
	Command.Event.Attach(Event.Buff.Add, function(_, unit, buffs) if analyzer:GetVisible() == true then analyzer:BuffAdd(unit, buffs) end end, "nkRebuff_Analyzer_BuffAdd")
	Command.Event.Attach(Event.Ability.New.Cooldown.Begin, function (_, cooldowns) if analyzer:GetVisible() == true then analyzer:CooldownBegin(cooldowns) end end, "nkRebuff_Analyzer_CooldownBegin")
	
	return analyzer
	
end