local addonInfo, privateVars = ...

privateVars.layout = {
	elementColor = { 0.925, 0.894, 0.741, 1 },
	labelColor = { 0.925, 0.894, 0.741, 1 }
}

function nkRebuff.Config ()

	local name = "nkRebuff.config"

	local config = EnKai.uiCreateFrame("nkWindowMetro", name, privateVars.uiElements.configContext)	
	local tabPane = EnKai.uiCreateFrame("nkTabpaneMetro", name .. ".TabPane", config:GetContent())
	
	local paneTabSettings = nkRebuff.ConfigTabSettings(tabPane)
	local paneTabTrackGrid = nkRebuff.ConfigTabTrackGrid(tabPane)
	local paneTabIconDefaults = nkRebuff.ConfigTabIconDefaults(tabPane)
	local paneTabExchange = nkRebuff.ConfigTabExchange(tabPane)
	local paneTabManual = nkRebuff.ConfigTabManual(tabPane)
		
	config:SetTitle(addonInfo.toc.Name)
	config:SetPoint("CENTER", UIParent, "CENTER")
	config:SetWidth(807)
	config:SetHeight(600)	
	
	tabPane:SetBorder(false)
  tabPane:SetPoint("TOPLEFT", config:GetContent(), "TOPLEFT", 10, 10)
  tabPane:SetPoint("BOTTOMRIGHT", config:GetContent(), "BOTTOMRIGHT", -10, -50)
		
	tabPane:AddPane( { label = privateVars.langTexts.tabHeaderSettings, frame = paneTabSettings, initFunc = function() paneTabSettings:build() end}, false)
	tabPane:AddPane( { label = privateVars.langTexts.tabHeaderIconDefaults, frame = paneTabIconDefaults, initFunc = function() paneTabIconDefaults:build() end}, false)
	tabPane:AddPane( { label = privateVars.langTexts.tabHeaderTracking, frame = paneTabTrackGrid, initFunc = function() paneTabTrackGrid:build() end}, false)
	tabPane:AddPane( { label = privateVars.langTexts.tabHeaderExchange, frame = paneTabExchange, initFunc = function() paneTabExchange:build() end}, false)
	tabPane:AddPane( { label = privateVars.langTexts.tabHeaderManual, frame = paneTabManual, initFunc = function() paneTabManual:build() end}, true)
	
	local EnKaiLogo = EnKai.uiCreateFrame("nkTexture", name .. ".EnKaiLogo", config)
  EnKaiLogo:SetTextureAsync(EnKai.art.GetThemeLogo()[1],EnKai.art.GetThemeLogo()[2])
  EnKaiLogo:SetPoint("BOTTOMLEFT", config:GetContent(), "BOTTOMLEFT", 10, -5)
  EnKaiLogo:SetWidth(125)
  EnKaiLogo:SetHeight(33)
  
  local closeButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".closeButton", config:GetContent())
  
  closeButton:SetPoint("BOTTOMRIGHT", config:GetContent(), "BOTTOMRIGHT", -10, -10)
  closeButton:SetText(privateVars.langTexts.btClose)
  closeButton:SetIcon("nkRebuff", "gfx/iconClose.png")
  closeButton:SetScale(.8)
  closeButton:SetLayer(9)
  
  Command.Event.Attach(EnKai.events[name .. ".closeButton"].Clicked, function (_, newValue)
    config:SetVisible(false) 
  end, name .. ".closeButton.Clicked")
 		
	config.setinfo = EnKai.uiCreateFrame('nkText', name ..'.setInfo', config:GetContent())
	config.setinfo:SetLayer(2)
	config.setinfo:SetFontSize(14)
	config.setinfo:SetPoint("BOTTOMLEFT", tabPane, "BOTTOMLEFT", 10, -5)
	config.setinfo:SetText(nkRBSets[nkRBConfig.activeSet].name)
		
	local analyzerButton = EnKai.uiCreateFrame('nkButtonMetro', name .. '.analyzerButton', config:GetContent())	
	analyzerButton:SetText(privateVars.langTexts.analyze)
	analyzerButton:SetPoint("CENTERRIGHT", closeButton, "CENTERLEFT", -5, 0)
	analyzerButton:SetIcon("nkRebuff", "gfx/iconScan.png")
  analyzerButton:SetScale(.8)
	analyzerButton:SetLayer(9)
	
	analyzerButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
		 if privateVars.uiElements.Analyzer == nil then 
			privateVars.uiElements.Analyzer = nkRebuff.Analyzer() 
		else
			privateVars.uiElements.Analyzer:ClearGrid()
		end
		privateVars.uiElements.Analyzer:SetVisible(true)
	end, 'nkRB.Config.analyzerButton.Left.Click')
	
	local oSetVisible = config.SetVisible
	
	function config:SetVisible(flag)
		if privateVars.isSecure == false then
			nkRebuff.buildReBuff()
			
			if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetDragable(flag) end
			if privateVars.uiElements.buffButtonBar ~= nil then privateVars.uiElements.buffButtonBar.subButtons[1]:SetDragable(flag) end
			
			if flag == false then 
				if nkRBConfig.buffButton ~= "0" then nkRebuff.setReBuffMacro() end
				if nkRBConfig.buffButton == "always" then 
					if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetVisible(true) end 
				else 
					if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetVisible(false) end 
				end
					
				if nkRBConfig.buffButton == "bar" then 
					if privateVars.uiElements.buffButtonBar ~= nil then privateVars.uiElements.buffButtonBar:SetVisible(true) end
				else
					if privateVars.uiElements.buffButtonBar ~= nil then privateVars.uiElements.buffButtonBar:SetVisible(false) end
				end
			else
				if nkRBConfig.buffButton == "bar" then
					if privateVars.uiElements.buffButtonBar ~= nil then privateVars.uiElements.buffButtonBar:SetVisible(true) end
				else
					if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetVisible(true) end
				end
			end
		end
		
		oSetVisible(self, flag)
		
		if flag == true then
			nkRebuff.displayAllIcons(true)
		else
			nkRebuff.hideAllIcons()
			nkRebuff.CheckMissingBuffs()
			nkRebuff.CheckMissingBuffsPet()
			nkRebuff.CheckMissingDebuffs()
		end
	end
	
	config:SetVisible(true)
	
	function tabPane:UpdateTrackGrid()
		-- this is set to tabPane so that other panes can access it by parent:UpdateTrackGrid()
		paneTabTrackGrid:Update()
	end
	
	function config:UpdateTrackGrid()
		-- this is used for updating from extern (analyzer)
		paneTabTrackGrid:Update()
	end
	
	function config:UpdateSetSelection()
		paneTabSettings:UpdateSetSelection()
	end
	
	local versionText = EnKai.uiCreateFrame("nkText", name .. ".versionText", config)
  versionText:SetFontSize(11)
  versionText:SetText(string.format(privateVars.langTexts.txtVersion, addonInfo.toc.Version))
  versionText:SetPoint("BOTTOMRIGHT", tabPane, "BOTTOMRIGHT", -5, -5)
  versionText:SetLayer(98)	
			
	return config
	
end