local addonInfo, privateVars = ...

function nkRebuff.ConfigTabExchange (parent)
	
	local name = parent:GetName() .. '.tabExchange'
	
	local tabPane = UI.CreateFrame ("Frame", name, parent)
	local editField, exportButton, importButton, exportHint
	
	function tabPane:build ()
		
		editField = EnKai.uiCreateFrame("nkTextfield", name .. '.editField', tabPane)
		exportButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".exportButton", tabPane)
		importButton = EnKai.uiCreateFrame("nkButtonMetro", name .. ".importButton", tabPane)
		
		exportButton:SetPoint("TOPLEFT", tabPane, "TOPLEFT")
		exportButton:SetText(privateVars.langTexts.exportButton)
		exportButton:SetWidth(200)
		exportButton:SetScale(.8)
		
		importButton:SetPoint("CENTERLEFT", exportButton, "CENTERRIGHT", 10, 0)
		importButton:SetText(privateVars.langTexts.importButton)
		importButton:SetWidth(200)
    importButton:SetScale(.8)
		
		editField:SetPoint("TOPLEFT", exportButton, "BOTTOMLEFT", 0, 30)
		editField:SetWidth(tabPane:GetWidth() - 24)
		editField:SetHeight(300)
		
		exportButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			tabPane:export()
		end, name .. '.exportButton.Left.Click')
		
		importButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			tabPane:import()
		end, name .. '.importButton.Left.Click')
		
		exportHint = EnKai.uiCreateFrame('nkInfoText', name .. '.exportHint', tabPane)
		exportHint:SetPoint("TOPRIGHT", tabPane, "TOPRIGHT")
		exportHint:SetFontSize(12)
		exportHint:SetWidth(400)
		exportHint:SetHeight(60)
		exportHint:SetType('info')
		exportHint:SetIconAlign('top')
		exportHint:SetText(privateVars.langTexts.exportHint)
				
	end
	
	function tabPane:import()
		local data = editField:GetText()
		
		local start = string.find(data, "!!")
		
		if start == nil then
			nkRebuff.messageDialog (privateVars.langTexts.importErrorFormat)
			return
		end
		
		local checksum = data:sub(1, start-1)
		local compressed = data:sub(start+2)
		
	  	if EnKai.zip.checksum(EnKai.zip.decodeBase64(compressed)) ~= checksum then
	  		nkRebuff.messageDialog (privateVars.langTexts.importErrorChecksum)
	  		return
	  	end
	  	
	  	local uncompressed = EnKai.zip.uncompress (compressed)
	  	
	  	local importTable = {}
	  	for entry in string.gmatch(uncompressed, "[^;]+") do
	  		table.insert(importTable, entry)
	  	end	  	
	  	
	  	local duplicateName = false
	  	
	  	for k, v in pairs (nkRBSets) do
	  		if v.name == importTable[1] then
	  			duplicateName = true
	  			break
	  		end	  		
	  	end
	  	
	  	if duplicateName == false then
	  		tabPane:storeImported(importTable, true)
	  	else
	  		local yesFunc = function () tabPane:storeImported(importTable, false) end
			local noFunc = function ()
				importTable[1] = string.format(privateVars.langTexts.genericSettings.copiedSetName, importTable[1]) 
				tabPane:storeImported(importTable, true) 
			end 
			
			nkRebuff.confirmDialog (string.format(privateVars.langTexts.messages.importDuplicateSet, importTable[1]), yesFunc, noFunc)
	  	end 
	  	
	end
	
	function tabPane:storeImported(importTable, createNewflag)
		
		if createNewflag == true then
			local noOfSets = #nkRBSets
			table.insert(nkRBSets, { name = importTable[1], role = 0, buffsPet = {}, buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {} })
			table.insert(nkRBIcons, { buffsPet = {}, buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {} })
			table.insert(nkRBData, { buffsPet = {}, buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {} })
			nkRebuff.activateSet(#nkRBSets)
			
		elseif createNewflag == false then
			-- overwrite existing set
		
			for idx = 1, #nkRBSets, 1 do
				if nkRBSets[idx].name == importTable[1] then
					nkRebuff.activateSet(idx)
					break
				end
			end
		end
		
		local sets, icons, data
		
		local setLoad = loadstring (importTable[2])
		sets = setLoad()
		
		for buffType, data in pairs(sets) do
			if buffType ~= 'name' and k ~= 'role' then				
				nkRBSets[nkRBConfig.activeSet][buffType] = data
			end
		end
						
		local dataLoad = loadstring (importTable[3])
		data = dataLoad()
		
		for buffType, details in pairs(data) do
			nkRBData[nkRBConfig.activeSet][buffType] = details
		end
		
		local iconsLoad = loadstring (importTable[4])
		icons = iconsLoad()
		
		for buffType, details in pairs(icons) do
			nkRBIcons[nkRBConfig.activeSet][buffType] = details
		end
		
		privateVars.uiElements.configUI:UpdateSetSelection()
	end
	
	function tabPane:export()
	
		local data = nkRBSets[nkRBConfig.activeSet].name
	
		data = data .. ";return {" .. EnKai.tools.table.serialize (nkRBSets[nkRBConfig.activeSet]) .. "}"
		data = data .. ";return {" .. EnKai.tools.table.serialize (nkRBData[nkRBConfig.activeSet]) .. "}"
		data = data .. ";return {" .. EnKai.tools.table.serialize (nkRBIcons[nkRBConfig.activeSet]) .. "}"
			
		local checksum, compressed = EnKai.zip.compress (data:gsub('\\', '/'))
		
		local lineTable = {}
		for idx = 1, compressed:len(), 100 do table.insert(lineTable, compressed:sub(idx, idx + 99)) end
		
		local exportString = table.concat(lineTable, "\n")
		editField:SetText(checksum .. "!!" .. exportString)		
		editField:SetKeyFocus(true)
		editField:SetSelection(1, string.len(checksum .. "!!" .. exportString))		
	end
	
	return tabPane

end