 local addonInfo, privateVars = ...

function nkRebuff.ConfigTabSettings (parent)

	local name = parent:GetName().. '.settings'

	local tabPane = UI.CreateFrame ("Frame", name, parent)
	local labelBasic
	local activeCheckbox
	local lockedCheckbox
	local labelFunctions
	local showOnlyInCombatCheckbox
	local autoRoleSwitchCheckbox
	local checkForUpdatesCheckbox
	local debuffWarningCheckbox
	local showTooltipsCheckbox
	local labelPerformance
	local throttleSlider
	local setSelectLabel
	local setSelect
	local setSelectHint
	local editIcon
	local editSetName
	local roleSelect
	local addButton
	local removeButton
	local copySetButton
	local labelBuffButton
	local bbDisplaySelect
	local bbScaleSlider
	local bbHint
	local usePluginCheckbox
	
	function tabPane:build ()
	
		labelBasic = EnKai.uiCreateFrame("nkText", name .. 'labelBasic', tabPane)
		labelBasic:SetPoint("TOPLEFT", tabPane, "TOPLEFT")
		labelBasic:SetEffectGlow({ offsetX = 2, offsetY = 2})
		labelBasic:SetText(privateVars.langTexts.genericSettings.basic)
		labelBasic:SetFontSize(16)
			
		activeCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. 'activeCheckbox', tabPane)	
		activeCheckbox:SetPoint("TOPLEFT", labelBasic, "BOTTOMLEFT", 0, 10)
		activeCheckbox:SetWidth(200)
		activeCheckbox:SetText(privateVars.langTexts.genericSettings.active)
		activeCheckbox:SetChecked(nkRBConfig.active)
		activeCheckbox:SetLabelInFront(true)
		
		Command.Event.Attach(EnKai.events[name .. 'activeCheckbox'].CheckboxChanged, function (_, newValue)		
			nkRBConfig.active = newValue
			if nkRebuff.buffBar ~= nil then nkRebuff.buffBar:SetVisible(nkRBConfig.active) end 
		end, name .. 'activeCheckbox.CheckboxChanged')
		
		usePluginCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. 'usePluginCheckbox', tabPane)	
		usePluginCheckbox:SetPoint("TOPLEFT", activeCheckbox, "BOTTOMLEFT", 0, 10)
		usePluginCheckbox:SetWidth(200)
		usePluginCheckbox:SetText(privateVars.langTexts.genericSettings.usePluginCheckbox)
		usePluginCheckbox:SetChecked(nkRBConfig.nkPanel)
		usePluginCheckbox:SetLabelInFront(true)
		
		Command.Event.Attach(EnKai.events[name .. 'usePluginCheckbox'].CheckboxChanged, function (_, newValue)		
			nkRBConfig.nkPanel = newValue
			if nkPanel ~= nil then
				if privateVars.uiElements.panel ~= nil then
					if nkPanel.api.togglePanel ~= nil then -- nkPanel V1.1.1 check, can be removed later
						nkPanel.api.togglePanel('nkRebuff', newValue)
					end
				elseif newValue == true then
					local items = {}
					for idx = 1, #nkRBSets, 1 do
						table.insert (items, { label = nkRBSets[idx].name, callBack = function() nkRebuff.activateSet (idx) end })
					end
					
					privateVars.uiElements.panel = nkRebuff.nkPanelPlugin(items)
					nkPanel.api.registerPlugin('nkRebuff', privateVars.uiElements.panel)
					privateVars.uiElements.panel:SetTitle(nkRBSets[nkRBConfig.activeSet].name)
				end
			end
		end, name .. 'usePluginCheckbox' ..".CheckboxChanged")	
		
		showOnlyInCombatCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. 'showOnlyInCombatCheckbox', tabPane)	
		showOnlyInCombatCheckbox:SetPoint("TOPLEFT", usePluginCheckbox, "BOTTOMLEFT", 0, 10)
		showOnlyInCombatCheckbox:SetWidth(200)
		showOnlyInCombatCheckbox:SetText(privateVars.langTexts.genericSettings.cbCombat)
		showOnlyInCombatCheckbox:SetChecked(nkRBConfig.combatOnly)
		showOnlyInCombatCheckbox:SetLabelInFront(true)
		
		Command.Event.Attach(EnKai.events[name .. 'showOnlyInCombatCheckbox'].CheckboxChanged, function (_, newValue)		
			nkRBConfig.combatOnly = newValue 
		end, name .. 'showOnlyInCombatCheckbox' ..".CheckboxChanged")
		
		autoRoleSwitchCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. 'autoRoleSwitchCheckbox', tabPane)	
		autoRoleSwitchCheckbox:SetPoint("TOPLEFT", showOnlyInCombatCheckbox, "BOTTOMLEFT", 0, 10)
		autoRoleSwitchCheckbox:SetWidth(200)
		autoRoleSwitchCheckbox:SetText(privateVars.langTexts.genericSettings.roleSwitch)
		autoRoleSwitchCheckbox:SetChecked(nkRBConfig.autoRoleSwitch)
		autoRoleSwitchCheckbox:SetLabelInFront(true)
				
		Command.Event.Attach(EnKai.events[name .. 'autoRoleSwitchCheckbox'].CheckboxChanged, function (_, newValue)		
			nkRBConfig.autoRoleSwitch = newValue 
			if newValue == true then 
				Command.Event.Attach(Event.Unit.Detail.Role, nkRebuff.RoleChange, "nkRebuff.Unit.Detail.Role")
			else
				Command.Event.Detach(Event.Unit.Detail.Role, nil, "nkRebuff.Unit.Detail.Role")
			end
		end, name .. 'autoRoleSwitchCheckbox' ..".CheckboxChanged")
		
		checkForUpdatesCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. 'checkForUpdatesCheckbox', tabPane)	
		checkForUpdatesCheckbox:SetPoint("TOPLEFT", autoRoleSwitchCheckbox, "BOTTOMLEFT", 0, 10)
		checkForUpdatesCheckbox:SetWidth(200)
		checkForUpdatesCheckbox:SetText(privateVars.langTexts.genericSettings.checkForUpdatesCheckbox)
		checkForUpdatesCheckbox:SetChecked(nkRBConfig.checkforupdates)
		checkForUpdatesCheckbox:SetLabelInFront(true)
		
		Command.Event.Attach(EnKai.events[name .. 'checkForUpdatesCheckbox'].CheckboxChanged, function (_, newValue)		
			nkRBConfig.checkforupdates = newValue 
		end, name .. 'checkForUpdatesCheckbox' ..".CheckboxChanged")	
		
		showTooltipsCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. 'showTooltipsCheckbox', tabPane)	
		showTooltipsCheckbox:SetPoint("TOPLEFT", checkForUpdatesCheckbox, "BOTTOMLEFT", 0, 10)
		showTooltipsCheckbox:SetWidth(200)
		showTooltipsCheckbox:SetText(privateVars.langTexts.genericSettings.showTooltipsCheckbox)
		showTooltipsCheckbox:SetChecked(nkRBConfig.showTooltips)
		showTooltipsCheckbox:SetLabelInFront(true)
			
		Command.Event.Attach(EnKai.events[name .. 'showTooltipsCheckbox'].CheckboxChanged, function (_, newValue)		
			nkRBConfig.showTooltips = newValue
			nkRebuff.displayAllIcons(true)
		end, name .. 'showTooltipsCheckbox' ..".CheckboxChanged")
	
		labelPerformance = EnKai.uiCreateFrame("nkText", name .. 'labelPerformance', tabPane)
		labelPerformance:SetPoint("TOPLEFT", showTooltipsCheckbox, "BOTTOMLEFT", 0, 25)
		labelPerformance:SetText(privateVars.langTexts.genericSettings.performance)
		labelPerformance:SetEffectGlow({ offsetX = 2, offsetY = 2})
		labelPerformance:SetFontSize(16)
		
		throttleSlider = EnKai.uiCreateFrame("nkSlider", name .. 'throttleSlider', tabPane)	
		throttleSlider:SetPoint("TOPLEFT", labelPerformance, "BOTTOMLEFT", 0, 10)
		throttleSlider:SetWidth(300)
		throttleSlider:SetRange(0.1, 1)
		throttleSlider:SetLabelWidth(190)
		throttleSlider:SetText(privateVars.langTexts.genericSettings.throttleSlider)
		throttleSlider:SetPrecision(0.1)
		throttleSlider:AdjustValue(nkRBConfig.throttle)
		
		Command.Event.Attach(EnKai.events[name .. 'throttleSlider'].SliderChanged, function (_, newValue)
			nkRBConfig.throttle = newValue
		end, name .. 'throttleSlider' .. ".SliderChanged")
		
		labelBuffButton = EnKai.uiCreateFrame("nkText", name .. 'labelBuffButton', tabPane)
		labelBuffButton:SetPoint("TOPLEFT", throttleSlider, "BOTTOMLEFT", 0, 25)
		labelBuffButton:SetEffectGlow({ offsetX = 2, offsetY = 2})
		labelBuffButton:SetText(privateVars.langTexts.genericSettings.buffButton)
		labelBuffButton:SetFontSize(16)
		labelBuffButton:SetFontColor(1, 1, 1, 1)	

		bbDisplaySelect = EnKai.uiCreateFrame("nkCombobox", name .. "bbDisplaySelect", tabPane)
		bbDisplaySelect:SetPoint("TOPLEFT", labelBuffButton, "BOTTOMLEFT", 0, 10)
		bbDisplaySelect:SetLayer(9)
		bbDisplaySelect:SetWidth(350)
		bbDisplaySelect:SetLabelWidth(190)
		bbDisplaySelect:SetSelection(privateVars.langTexts.genericSettings.bbDisplay)
		bbDisplaySelect:SetSelectedValue(nkRBConfig.buffButton, false)
		bbDisplaySelect:SetText(privateVars.langTexts.genericSettings.showButton)
		
		Command.Event.Attach(EnKai.events[name .. "bbDisplaySelect"].ComboChanged, function (_, newValue)
			
			nkRBConfig.buffButton = newValue.value
			
			if newValue.value  ~= '0' then 
				nkRebuff.buildReBuff()
				nkRebuff.setReBuffMacro(false)
				--privateVars.uiElements.buffButton:SetVisible(true)				
			else
				if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetVisible(false) end
			end
		end, name .. "parent:UpdateTrackGrid().ComboChanged")
		
		bbHint = EnKai.uiCreateFrame('nkInfoText', name .. '.bbHint', tabPane)
		bbHint:SetPoint("TOPLEFT", bbDisplaySelect, "TOPRIGHT", 5, 0)
		bbHint:SetFontSize(10)
		bbHint:SetWidth(400)
		bbHint:SetHeight(60)
		bbHint:SetType('info')
		bbHint:SetIconAlign('top')
		bbHint:SetText(privateVars.langTexts.genericSettings.bbHint)
		
		bbScaleSlider = EnKai.uiCreateFrame("nkSlider", name .. 'bbScaleSlider', tabPane)	
		bbScaleSlider:SetPoint("TOPLEFT", bbDisplaySelect, "BOTTOMLEFT", 0, 10)
		bbScaleSlider:SetWidth(300)
		bbScaleSlider:SetRange(20, 150)
		bbScaleSlider:SetLabelWidth(190)
		bbScaleSlider:SetText(privateVars.langTexts.scale)
		bbScaleSlider:SetPrecision(0.1)
		bbScaleSlider:AdjustValue(nkRBConfig.buffButtonScale)
		
		Command.Event.Attach(EnKai.events[name .. 'bbScaleSlider'].SliderChanged, function (_, newValue)
			nkRBConfig.buffButtonScale = newValue
			
			if Inspect.System.Secure() == true then return end
			
			if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetScale(nkRBConfig.buffButtonScale/100) end
			if privateVars.uiElements.buffButtonBar ~= nil then
				local width = 0
				for idx = 1, #privateVars.uiElements.buffButtonBar.subButtons, 1 do				  
					privateVars.uiElements.buffButtonBar.subButtons[idx]:SetScale(nkRBConfig.buffButtonScale/100)
					width = width + privateVars.uiElements.buffButtonBar.subButtons[idx]:GetWidth()
				end 
				
				privateVars.uiElements.buffButtonBar:SetHeight(privateVars.uiElements.buffButtonBar.subButtons[1]:GetHeight())
				privateVars.uiElements.buffButtonBar:SetWidth(width)
			end			
		end, name .. 'bbScaleSlider' .. ".SliderChanged")
		
		-- SET MANAGEMENT
				
		setSelectLabel = EnKai.uiCreateFrame("nkText", name .. 'setSelectLabel', tabPane)
		setSelectLabel:SetPoint("TOPLEFT", tabPane, "TOPLEFT", 320, 0)
		setSelectLabel:SetText(privateVars.langTexts.genericSettings.sets)
		setSelectLabel:SetEffectGlow({ offsetX = 2, offsetY = 2})
		setSelectLabel:SetFontSize(16)
		
		local selection = {}
		
		for idx = 1, #nkRBSets, 1 do
			table.insert (selection, { label = nkRBSets[idx].name, value = idx })		
		end
		
		setSelect = EnKai.uiCreateFrame("nkCombobox", name .. "setSelect", tabPane)
		setSelect:SetPoint("TOPLEFT", setSelectLabel, "BOTTOMLEFT", 0, 10)
		setSelect:SetLayer(9)
		setSelect:SetWidth(350)
		setSelect:SetLabelWidth(150)
		setSelect:SetSelection(selection)
		setSelect:SetSelectedValue(nkRBConfig.activeSet, false)
		setSelect:SetText(privateVars.langTexts.genericSettings.currentSet)
		
		Command.Event.Attach(EnKai.events[name .. "setSelect"].ComboChanged, function (_, newValue)
			nkRebuff.activateSet(newValue.value)
			roleSelect:SetSelectedValue(nkRBSets[newValue.value].role)
			parent:UpdateTrackGrid()
		end, name .. "setSelect.ComboChanged")
		
		editIcon = UI.CreateFrame('Texture', name .. 'editIcon', tabPane)
		editIcon:SetTextureAsync("nkRebuff", "gfx/editPen.png")	
		editIcon:SetWidth(15)
		editIcon:SetHeight(15)
		editIcon:SetPoint("CENTERLEFT", setSelect, "CENTERRIGHT", 5, 0)
		
		editIcon:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			if editSetName:GetVisible() == true then
				editSetName:SetKeyFocus(false)
				setSelect:SetComboVisible(true)
				editSetName:SetVisible(false)
			else			
				editSetName:SetText(setSelect:GetSelectedLabel())
				setSelect:SetComboVisible(false)
				editSetName:SetVisible(true)
			end
		end, name .. 'editIcon_LeftClick')
		
		editSetName = EnKai.uiCreateFrame("nkTextfield", name .. 'editSetName', tabPane)	
		editSetName:SetVisible(false)
		editSetName:SetWidth(200)
		editSetName:SetHeight(20)
		editSetName:SetText(nkRBSets[1].name)
		editSetName:SetPoint("TOPRIGHT", setSelect, "TOPRIGHT")
		
		Command.Event.Attach(EnKai.events[name .. 'editSetName'].TextfieldChanged, function ()		
			nkRBSets[setSelect:GetSelectedValue()].name = editSetName:GetText()
			tabPane:UpdateSetSelectValues()
			setSelect:SetComboVisible(true)
			editSetName:SetVisible(false)
		end, name .. 'editSetName.TextfieldChanged')
		
		setSelectHint = EnKai.uiCreateFrame('nkInfoText', name .. '.setSelectHint', tabPane)
		setSelectHint:SetPoint("TOPRIGHT", setSelect, "BOTTOMRIGHT", 0, 5)
		setSelectHint:SetFontSize(10)
		setSelectHint:SetWidth(setSelect:GetWidth())
		setSelectHint:SetHeight(30)
		setSelectHint:SetType('info')
		setSelectHint:SetText(privateVars.langTexts.genericSettings.setsHint)
		
		local selValues = privateVars.langTexts.genericSettings.selectionRole
		--local roleList = Inspect.Role.List()
		
		local roleList = Inspect.Role.List()
		for k, v in pairs (roleList) do
			local id = string.sub ( k, string.len(k) - 1)
			if id ~= nil then
				table.insert(selValues, { label = v, value = tonumber(id, 16)+1})
			end		
		end	
		
		roleSelect = EnKai.uiCreateFrame("nkCombobox", name .. "roleSelect", tabPane)
		roleSelect:SetPoint("TOPRIGHT", setSelectHint, "BOTTOMRIGHT", 0, 5)
		roleSelect:SetLayer(8)
		roleSelect:SetWidth(350)
		roleSelect:SetLabelWidth(150)
		roleSelect:SetText(privateVars.langTexts.genericSettings.role)
		roleSelect:SetSelection(selValues)
		roleSelect:SetSelectedValue(nkRBSets[nkRBConfig.activeSet].role, false)
		
		Command.Event.Attach(EnKai.events[name .. "roleSelect"].ComboChanged, function (_, newValue)
			nkRBSets[setSelect:GetSelectedValue()].role = newValue.value
		end, name .. "roleSelect.ComboChanged")
		
		addButton = EnKai.uiCreateFrame('nkButtonMetro', name .. 'addButton', tabPane)
		addButton:SetText(privateVars.langTexts.genericSettings.addButton)
		addButton:SetPoint("TOPLEFT", roleSelect, "BOTTOMLEFT", 0, 10)
		addButton:SetWidth(150)
		addButton:SetScale(.8)
		
		addButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			tabPane:AddSet()
		end, name .. "removeButton_LeftClick")	
		
		removeButton = EnKai.uiCreateFrame('nkButtonMetro', name .. 'removeButton', tabPane)
		removeButton:SetText(privateVars.langTexts.genericSettings.removeButton)
		removeButton:SetPoint("CENTERLEFT", addButton, "CENTERRIGHT", 5, 0)
		removeButton:SetWidth(150)
    removeButton:SetScale(.8)
		
		removeButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			tabPane:RemoveSet()
		end, name .. "removeButton_LeftClick")	
		
		copySetButton = EnKai.uiCreateFrame('nkButtonMetro', name .. 'copySetButton', tabPane)
		copySetButton:SetText(privateVars.langTexts.genericSettings.copySetButton)
		copySetButton:SetPoint("CENTERLEFT", removeButton, "CENTERRIGHT", 5, 0)
		copySetButton:SetWidth(150)
    copySetButton:SetScale(.8)

		copySetButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			tabPane:CopySet()
		end, name .. "copySetButton_LeftClick")	
	
	end
	
	---------- update set selection combo ----------
	
	function tabPane:UpdateSetSelectValues()

		local selection = {}
		
		for idx = 1, #nkRBSets, 1 do
			table.insert (selection, { label = nkRBSets[idx].name, value = idx })		
		end
		
		setSelect:SetSelection(selection)
		
	end
	
	function tabPane:AddSet()
		local noOfSets = #nkRBSets
		table.insert(nkRBSets, { name = string.format(privateVars.langTexts.genericSettings.newSetName, noOfSets+1), role = 0, buffsPet = {}, buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {} })
		table.insert(nkRBIcons, { buffsPet = {}, buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {} })
		table.insert(nkRBData, { buffsPet = {}, buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {} })
		tabPane:UpdateSetSelectValues()
		setSelect:SetSelectedValue(#nkRBSets)
		nkRebuff.activateSet(#nkRBSets)
	end
	
	function tabPane:RemoveSet()
		if #nkRBSets == 1 then 
			nkRebuff.messageDialog(privateVars.langTexts.messages.cannotRemoveLastSet)
		else
			local funcYes = function () 
				table.remove (nkRBSets, setSelect:GetSelectedValue())			
				table.remove (nkRBIcons, setSelect:GetSelectedValue())
				table.remove (nkRBData, setSelect:GetSelectedValue())
				tabPane:UpdateSetSelectValues()
				setSelect:SetSelectedValue(1)
				nkRebuff.activateSet(1)
			end
			
			nkRebuff.confirmDialog (privateVars.langTexts.messages.removeCurrentSet, funcYes)
		end
	
	end
	
	function tabPane:CopySet()
		local noOfSets = #nkRBSets
		local newSet = EnKai.tools.table.copy (nkRBSets[setSelect:GetSelectedValue()])
		
		newSet.name = string.format(privateVars.langTexts.genericSettings.copiedSetName, nkRBSets[setSelect:GetSelectedValue()].name)
		
		local newData = EnKai.tools.table.copy (nkRBData[setSelect:GetSelectedValue()])
		local newIcons = EnKai.tools.table.copy (nkRBIcons[setSelect:GetSelectedValue()])
		
		table.insert ( nkRBSets, newSet)
		table.insert ( nkRBIcons, newIcons)
		table.insert ( nkRBData, newData )
		
		tabPane:UpdateSetSelectValues()		
		setSelect:SetSelectedLabel (newSet.name)
		nkRebuff.activateSet(#nkRBSets)
	end
	
	function tabPane:UpdateSetSelection()
		tabPane:UpdateSetSelectValues()
		setSelect:SetSelectedValue(nkRBConfig.activeSet)
	end	
	
	return tabPane
	
end