local addonInfo, privateVars = ...

function nkRebuff.ConfigTabTrackGrid (parent)

	local name = parent:GetName().. '.trackGrid'

	local tabPane = UI.CreateFrame ("Frame", name, parent)
	local gridBG
	local grid
	local slider
	local settingLabel
	local settingSelect
	local useableCheckbox
	local activeOnlyCheckbox
	local hint
	local removeButton
	
	local iconEdit
	local isBuilt = false
	
	local gridRows = 19
	
	function tabPane:build ()
	
	   -- setting selection
      
    settingSelect = EnKai.uiCreateFrame("nkCombobox", name .. 'settingSelect', tabPane)
    settingSelect:SetPoint("TOPLEFT", tabPane, "TOPLEFT")
    settingSelect:SetWidth(300)
    settingSelect:SetSelection(privateVars.langTexts.settingSelection)
    settingSelect:SetSelectedValue(privateVars.langTexts.settingSelection[1].value, false)
    settingSelect:SetText(privateVars.langTexts.selSetting)
    settingSelect:SetLabelWidth(100)
    settingSelect:SetLayer(9)
    
    Command.Event.Attach(EnKai.events[name .. 'settingSelect'].ComboChanged, function (_, newValue)
      if grid:GetSelectedRow() ~= nil then grid:RowSelect(grid:GetSelectedRow(), false) end
      tabPane:PopulateGrid(newValue.value)      
    end, name .. 'settingSelect.ComboChanged')
	
		-- grid definition
		
		grid = EnKai.uiCreateFrame("nkGrid", name .. 'grid', tabPane)
		
		grid:SetHeaderHeight(25)
		grid:SetPoint("TOPLEFT", settingSelect, "BOTTOMLEFT", 0, 35)
		grid:SetHeaderHeight(0)
		grid:SetSelectable(true)
		grid:SetLayer(2)
		
		local cols = {	{ align = 'center', texture = true, textureType = 'nkRebuff', textureHeight = 14, textureWidth = 14, width = 20,  }, 
						{ align = 'center', texture = true, textureType = 'Rift', width = 30}, 
						{ align = 'left', width = 300 } }	
		
		grid:SetVisible(false)	
		grid:Layout (cols, gridRows)	
									
		Command.Event.Attach(EnKai.events[name .. 'grid'].GridFinished, function ()
			tabPane:PopulateGrid(privateVars.langTexts.settingSelection[1].value) 
			grid:SetVisible(true)
		end, name .. 'grid.GridFinished')
		
		Command.Event.Attach(EnKai.events[name .. 'grid'].WheelForward, function (_, rowPos)			
			slider:AdjustValue (rowPos)
		end, name .. 'grid.Grid.WheelForward')
		
		Command.Event.Attach(EnKai.events[name .. 'grid'].WheelBack, function (_, rowPos)
			slider:AdjustValue (rowPos)
		end, name .. 'grid.Grid.WheelBack')
		
		Command.Event.Attach(EnKai.events[name .. 'grid'].LeftClick, function (_, row)
			if grid:GetKey(row) == nil then return end
			tabPane:SetIconEdit(grid:GetKey(row), grid:GetCellValue(row, 2), grid:GetCellValue(row, 3))
		end, name .. 'grid.LeftClick')				
		
		Command.Event.Attach(EnKai.events[name .. 'grid'].RightClick, function (_, row)
			
			if nkRBConfig.activeSet == nil then
				EnKai.tools.error.display (addonInfo.toc.Identifier, "There is no active set!", 1) 
				return 
			end
			
			if grid:GetKey(row) == nil then return end
			
			local cellValue = grid:GetCellValue(row, 1)
			local active = false
			
			if cellValue == 'gfx/statusRed.png' then
				grid:SetCellValue(row, 1, 'gfx/statusGreen.png')
				active = true
			else
				grid:SetCellValue(row, 1, 'gfx/statusRed.png')
				active = false
			end
			
			if settingSelect:GetSelectedValue() == nil then
				EnKai.tools.error.display (addonInfo.toc.Identifier, "Ability selection reports no value selected", 1)
				return
			elseif grid:GetKey(row) == nil then
				EnKai.tools.error.display (addonInfo.toc.Identifier, string.format("Cannot get key for row #%d", row), 1)
				return
			end
			
			if nkRBSets[nkRBConfig.activeSet][settingSelect:GetSelectedValue()][grid:GetKey(row)] == nil then
				EnKai.tools.error.display (addonInfo.toc.Identifier, string.format("No set information for combination %d, %s, %s!", nkRBConfig.activeSet, settingSelect:GetSelectedValue(), grid:GetKey(row)), 1) 
				return
			end
			
			nkRBSets[nkRBConfig.activeSet][settingSelect:GetSelectedValue()][grid:GetKey(row)].active = active
			
		end, name .. 'grid.RightClick')
		
		-- grid slider
		
		slider = EnKai.uiCreateFrame("nkScrollbox", name .. "slider", tabPane)	
		
		slider:SetPoint("TOPLEFT", grid, "TOPRIGHT")
		slider:SetHeight(grid:GetHeight())
		slider:SetRange(1, 1)
		slider:SetVisible(false)
		slider:SetLayer(2)
		
		Command.Event.Attach(EnKai.events[name .. "slider"].ScrollboxChanged, function ()		
			grid:SetRowPos(math.floor(slider:GetValue('value')), true)
		end, name .. 'slider.ScrollboxChanged')

    -- hint
    
    hint = EnKai.uiCreateFrame('nkText', name ..'.hint', tabPane)
    hint:SetText(privateVars.langTexts.trackGridHint)
    hint:SetFontSize(11)
    hint:SetPoint("BOTTOMCENTER", grid, "TOPCENTER")
    hint:SetLayer(2)
    
		iconEdit = nkRebuff.IconEdit (tabPane)
		iconEdit:SetPoint("TOPRIGHT", tabPane, "TOPRIGHT", 0, 0)
		iconEdit:SetVisible(false)
		
		removeButton = UI.CreateFrame ('RiftButton', name .. '.removeButton', tabPane)	
		removeButton:SetText(privateVars.langTexts.removeAbility)
		removeButton:SetPoint("BOTTOMLEFT", parent:GetParent(), "BOTTOMLEFT", 175, -12)
		removeButton:SetLayer(2)
		
		removeButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
			if grid:GetSelectedRow() == nil then return end
			local key = grid:GetKey(grid:GetSelectedRow())
			if key == nil then return end
			
			local buffType = settingSelect:GetSelectedValue()
			
			if privateVars.uiElements.icons[buffType][key] ~= nil then
				--privateVars.uiElements.icons[buffType][key]:SetVisible(false)
				privateVars.uiElements.icons[buffType][key]:Recycle()
				privateVars.uiElements.icons[buffType][key] = nil
			end
			
			nkRBSets[nkRBConfig.activeSet][buffType][key] = nil
			nkRBData[nkRBConfig.activeSet][buffType][key] = nil
			nkRBIcons[nkRBConfig.activeSet][buffType][key] = nil
			
			grid:RowSelect(grid:GetSelectedRow(), false)
			iconEdit:SetVisible(false)
			
			tabPane:PopulateGrid(buffType)	
			
		end, name .. '.removeButton.Left.Click')
		
		isBuilt = true
	end
																
	function tabPane:PopulateGrid (buffType)
	
		local setInfo = nkRBSets[nkRBConfig.activeSet][buffType]
		local abilityData = nkRBData[nkRBConfig.activeSet][buffType]
		
		local keyTable = {}
		
		for k, v in pairs(abilityData) do
			if v.name == nil then -- got an error report indicating that some buffs don't have a name ?!?
				keyTable[k] = k
			else
				keyTable[v.name] = k
			end
		end
		
		table.sort(keyTable, function (a, b) return string.lower(a) < string.lower(b) end)
		
		local gridValues = {}
		
		for name, id in pairs (keyTable) do
			
			if setInfo[id] == nil then 
				setInfo[id] = {}
				setInfo[id].active = false 
			end
	
			local status = 'gfx/statusRed.png'
			if setInfo[id].active == true then status = 'gfx/statusGreen.png' end
			
			if abilityData[id].name ~= nil then -- check if name is set. Player was reporting an error where name was nil on line 199 
				table.insert (gridValues, {{ key = id, value = status }, { value = abilityData[id].icon}, {value = abilityData[id].name} })
			end
		end
		
		table.sort (gridValues, function (v1, v2) return v1[3].value < v2[3].value end )
		
		grid:SetRowPos(1, false)
		grid:SetCellValues(gridValues)

		if (#gridValues-gridRows) > 0 then		
			slider:SetRange (1, #gridValues-gridRows+1)
			slider:AdjustValue(1)
			slider:SetVisible(true)			
		else
			slider:SetVisible(false)
		end
		
	end
	
	function tabPane:SetIconEdit(key, texturePath, label)
		iconEdit:SetBuffType(settingSelect:GetSelectedValue())
		iconEdit:SetKey(key)
		
		local storedIconData = nkRBIcons[nkRBConfig.activeSet][settingSelect:GetSelectedValue()][key]
		
		if storedIconData == nil then
			local iconData = EnKai.tools.table.copy (nkRBConfig.iconDefaults)
			iconData.texturePath = texturePath
			iconData.label = label
			
			iconEdit:RemoveIcon()
			iconEdit:SetConfig(iconData)
		else			
			-- evtl nur machen wenn auch aktiviert

			if privateVars.uiElements.icons[settingSelect:GetSelectedValue()][key] == nil then
				privateVars.uiElements.icons[settingSelect:GetSelectedValue()][key] = nkRebuff.createIcon (settingSelect:GetSelectedValue(), key, storedIconData)
				privateVars.uiElements.icons[settingSelect:GetSelectedValue()][key]:SetVisible(true)
			end
			
			iconEdit:SetIcon()
			iconEdit:SetConfig(storedIconData)
		end
			
		iconEdit:SetVisible(true)
	end
	
	function tabPane:Update(imageListOnly)
		-- updates configuration values
		if isBuilt == false then return end

		if imageListOnly ~= true then
			tabPane:PopulateGrid(settingSelect:GetSelectedValue()) 
			
			if grid:GetKey(1) ~= nil then
				tabPane:SetIconEdit(grid:GetKey(1), grid:GetCellValue(1, 2), grid:GetCellValue(1, 3))				
			else
				iconEdit:SetVisible(false)
			end
		end
		iconEdit:Update()
	end
							
	return tabPane
	
end
