local addonInfo, privateVars = ...

privateVars.effects = {
	gloss = { alpha = 0.6, texturePath = 'gfx/gloss.png', replaceBorder = false },
	round = { alpha = 0.6, texturePath = 'gfx/round.png', replaceBorder = false },
	basic = { alpha = 0.5, texturePath = 'gfx/basic.png', replaceBorder = false },
	chrome = { alpha = 1, texturePath = 'gfx/chrome.png', replaceBorder = true, border = 7},
	roundedCorners = { alpha = 1, texturePath = 'gfx/roundedCorners.png', replaceBorder = true, border = 1},
}

function nkRebuff.icon (name, parent)

	local icon = UI.CreateFrame ('Frame', name, parent)
	
	local border = UI.CreateFrame ('Frame', name, icon)
	local texture = UI.CreateFrame ('Texture', name .. '.texture', icon)
	local effect = UI.CreateFrame ('Texture', name .. '.effect', icon)
	
	local timer = UI.CreateFrame ("Text", name ..'.timer', icon)
	local stack = UI.CreateFrame ("Text", name ..'.stack', icon)
	
	local label = UI.CreateFrame ('Text', name .. 'label', icon)
	
	local properties = {}

	function icon:SetValue(property, value)
		properties[property] = value
	end
	
	function icon:GetValue(property)
		return properties[property]
	end
	
	icon:SetValue("name", name)
	icon:SetValue("parent", parent)
	
	icon:SetMouseMasking('limited')
	
	local dragable = false
	local showTooltip = false
	local timerFontSize = 24
	local stackFontSize = 18
	local labelFontSize = 16
	local showBorder = true
	local activeEffect = 'none'
	local scale = 1
		
	icon:SetWidth(50)
	icon:SetHeight(65)
	icon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", 100, 100)
	
	border:SetPoint("TOPLEFT", icon, "TOPLEFT")
	border:SetWidth(50)
	border:SetHeight(50)
	border:SetBackgroundColor(0, 0, 0, 1)
	border:SetLayer(1)
	
	texture:SetPoint("TOPLEFT", border, "TOPLEFT", 1, 1)
	texture:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", -1, -1)
	texture:SetLayer(2)
	
	timer:SetLayer(4)
	timer:SetFontColor(1, 1, 1, 0)
	timer:SetFontSize(timerFontSize)
	timer:SetHeight(timerFontSize+4)
	timer:SetPoint("CENTER", border, "CENTER")
	
	stack:SetLayer(4)
	stack:SetFontColor(1, 1, 1, 0)
	stack:SetFontSize(stackFontSize)
	stack:SetPoint("TOPLEFT", icon, "TOPLEFT", 0 , -1)
		
	label:SetPoint("TOPCENTER", border, "BOTTOMCENTER")
	label:SetFontColor (1, 1, 1, 1)
	label:SetFontSize(labelFontSize)
	label:SetHeight(labelFontSize+4)
	
	effect:SetLayer(3)
	effect:SetVisible(false)
	
	function icon:SetTexture(textureType, texturePath)
		texture:SetTextureAsync(textureType, texturePath)
	end
	
	function icon:SetLabel(text)
		label:ClearWidth()
		label:SetText(text)
	end
	
	function icon:SetTimer(text)
		timer:ClearWidth()
		timer:SetText(text)
	end
	
	function icon:SetStack(text)
		stack:ClearWidth()
		if text == nil then		
			stack:SetText("")
		else
			stack:SetText(tostring(text))
		end
	end
	
	function icon:ShowLabel(flag)
		label:SetVisible(flag)
	end
	
	function icon:ShowTimer(flag)
		timer:SetVisible(flag)
	end
	
	function icon:ShowStack(flag)
		stack:SetVisible(flag)
	end
	
	function icon:ShowBorder(flag)
	
		showBorder = flag
		
		effect:ClearAll()
		texture:ClearAll()
	
		if flag == true then
			texture:SetPoint("TOPLEFT", border, "TOPLEFT", 1, 1)
			texture:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", -1, -1)
			
			effect:SetPoint("TOPLEFT", border, "TOPLEFT", 1, 1)
			effect:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", -1, -1)
		else
			texture:SetPoint("TOPLEFT", border, "TOPLEFT", 0, 0)
			texture:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", 0, 0)
			
			effect:SetPoint("TOPLEFT", border, "TOPLEFT", 0, 0)
			effect:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", 0, 0)
		end
	end
	
	function icon:SetScale(newScale)
		scale = newScale
	
		icon:SetWidth( 50 * newScale)
		icon:SetHeight( 50 * newScale)
		
		border:SetWidth( 50 * newScale)
		border:SetHeight( 50 * newScale)
		
		local fontSize = math.floor(labelFontSize * newScale)
		
		label:SetFontSize(fontSize)
		label:SetHeight(fontSize + 4)
		
		local fontSize = math.floor(timerFontSize * newScale)
		
		timer:SetFontSize(fontSize)
		timer:SetHeight(fontSize + 4)
		
		local fontSize = math.floor(stackFontSize * newScale)
		
		stack:SetFontSize(fontSize )
		--stack:SetHeight(fontSize + 4)
	end
	
	function icon:SetLabelColor(r, g, b, a)
		label:SetFontColor(r, g, b, a)
	end
	
	function icon:SetTimerColor(r, g, b, a, ro, go, bo)
		timer:SetFontColor(r, g, b, a)
		timer:SetEffectGlow({ strength = 5, offsetX = 0, offsetY = 0, blurX = 3, blurY = 3, colorR = ro, colorG = go, colorB = bo })
	end
	
	function icon:SetStackColor(r, g, b, a)
		stack:SetFontColor(r, g, b, a)
		stack:SetEffectGlow({ offsetX = 1, offsetY = 1})
	end
	
	function icon:SetBorderColor(r, g, b, a)
		border:SetBackgroundColor(r, g, b, a)
	end
	
	function icon:SetInfoBlock (header, desc)
		icon:SetValue('infoBlockHeader', header)
		icon:SetValue('infoBlockDesc', desc)
	end
	
	function icon:SetDragable(flag)
	
		if flag == false then 
		
 			if dragable == true then
				icon:EventDetach(Event.UI.Input.Mouse.Left.Down, nil, name .. ".Left.Down")
				icon:EventDetach(Event.UI.Input.Mouse.Cursor.Move, nil, name .. ".Cursor.Move")
				icon:EventDetach(Event.UI.Input.Mouse.Left.Up, nil, name .. ".Left.Up")
			end
			
		else
		
			icon:EventAttach(Event.UI.Input.Mouse.Left.Down, function (self)		
			
				if dragable == false then return end
				
				self.leftDown = true
				local mouse = Inspect.Mouse()
				
				self.originalXDiff = mouse.x - self:GetLeft()
				self.originalYDiff = mouse.y - self:GetTop()
				
				local left, top, right, bottom = icon:GetBounds()
				
				icon:ClearAll()
				icon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", left, top)
				icon:SetWidth(right-left)
				icon:SetHeight(bottom-top)
			
			end, name .. ".Left.Down")
				
			icon:EventAttach( Event.UI.Input.Mouse.Cursor.Move, function (self, _, x, y)	
				if self.leftDown ~= true then return end
				icon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", x - self.originalXDiff, y - self.originalYDiff)
			end, name .. ".Cursor.Move")
			
			icon:EventAttach( Event.UI.Input.Mouse.Left.Up, function (self)	
				if self.leftDown ~= true then return end
			    self.leftDown = false
				
				local thisIcon = nkRBIcons[nkRBConfig.activeSet][icon:GetValue('buffType')][icon:GetValue('key')]
				
				thisIcon.ypos = icon:GetTop()
				thisIcon.xpos = icon:GetLeft()		
				
				for iconType, iconList in pairs(nkRBIcons[nkRBConfig.activeSet]) do
					local snapped = false
				
					for id, iconDef in pairs(iconList) do
						if iconType ~= icon:GetValue('iconType') and id ~= icon:GetValue('key') then
					
							local thatIcon = privateVars.uiElements.icons[iconType][id]				
							
							if thatIcon ~= nil then					
								if	thisIcon.xpos > thatIcon:GetLeft() - icon:GetWidth() - 20 and thisIcon.xpos < thatIcon:GetLeft() + thatIcon:GetWidth() + 20 and
									thisIcon.ypos > thatIcon:GetTop() - icon:GetHeight() - 20 and thisIcon.ypos < thatIcon:GetTop() + thatIcon:GetHeight() + 20 then
									-- snap it
									
									if thisIcon.xpos < thatIcon:GetLeft() then
										thisIcon.xpos = thatIcon:GetLeft() - icon:GetWidth()
										thisIcon.ypos = thatIcon:GetTop() + (thatIcon:GetHeight() / 2) - (icon:GetHeight() / 2)
									elseif thisIcon.xpos > thatIcon:GetLeft() + thatIcon:GetWidth() then
										thisIcon.xpos = thatIcon:GetLeft() + thatIcon:GetWidth()
										thisIcon.ypos = thatIcon:GetTop() + (thatIcon:GetHeight() / 2) - (icon:GetHeight() / 2)
									elseif thisIcon.ypos < thatIcon:GetTop() then
										thisIcon.xpos = thatIcon:GetLeft() + (thatIcon:GetWidth() / 2) - (icon:GetWidth() / 2)
										thisIcon.ypos = thatIcon:GetTop() - icon:GetHeight()
									else
										thisIcon.xpos = thatIcon:GetLeft() + (thatIcon:GetWidth() / 2) - (icon:GetWidth() / 2)
										thisIcon.ypos = thatIcon:GetTop() + thatIcon:GetHeight()
									end
									
									local left, top, right, bottom = icon:GetBounds()
									icon:ClearAll()
									icon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", thisIcon.xpos, thisIcon.ypos)
									icon:SetWidth(right-left)
									icon:SetHeight(bottom-top)
									
									snapped = true
									break
								end
							end
						end
					end
					
					if snapped == true then break end
				end
				
				privateVars.eventHandlers[name]["Moved"]()
			end, name .. ".Left.Up")
		end
		
		dragable = flag		
		
	end
	
	function icon:SetTooltip (flag)
		
		if flag == false then
			if showTooltip == true then		
				icon:EventDetach(Event.UI.Input.Mouse.Cursor.In, nil, name .. ".Mouse.Cursor.In")
				icon:EventDetach(Event.UI.Input.Mouse.Cursor.Out, nil, name .. ".Mouse.Cursor.Out")
			end
		else
			icon:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
				if dragable == true then return end
				
				if nkRBConfig.showTooltips == false then return end
			
				if privateVars.uiElements.infoBlock == nil then privateVars.uiElements.infoBlock = nkRebuff.infoBlock() end
				
				local header = icon:GetValue('infoBlockHeader')
				local desc = icon:GetValue('infoBlockDesc')
				
				if header == nil or desc == nil then return end
				
				privateVars.uiElements.infoBlock:SetHeader(header)
				privateVars.uiElements.infoBlock:SetDescription(desc)		
				
				local width, height = privateVars.uiElements.infoBlock:GetWidth(), privateVars.uiElements.infoBlock:GetHeight() 
				privateVars.uiElements.infoBlock:ClearAll()
				privateVars.uiElements.infoBlock:SetWidth(width)
				privateVars.uiElements.infoBlock:SetHeight(height)
				privateVars.uiElements.infoBlock:SetPoint("BOTTOMLEFT", icon, "TOPLEFT")
				privateVars.uiElements.infoBlock:SetVisible(true)
			end, name .. '.Mouse.Cursor.In')
			
			icon:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
				if privateVars.uiElements.infoBlock ~= nil then privateVars.uiElements.infoBlock:SetVisible(false) end
			end, name .. '.Mouse.Cursor.Out')
		end
		
		showTooltip = flag
		
	end
		
	function icon:SetEffect(newEffect)
		activeEffect = newEffect
	
		if newEffect == nil then
			effect:SetVisible(false)
			return
		end			
	
		effect:SetTextureAsync("nkRebuff", newEffect.texturePath)
		effect:SetAlpha(newEffect.alpha)
		
		effect:ClearAll()
		texture:ClearAll()
		
		if newEffect.replaceBorder == false and showBorder == true then
			effect:SetPoint("TOPLEFT", border, "TOPLEFT", 1, 1)
			effect:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", -1, -1)
			
			texture:SetPoint("TOPLEFT", border, "TOPLEFT", 1, 1)
			texture:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", -1, -1)
			
			border:SetVisible(true)
		else
			effect:SetPoint("TOPLEFT", border, "TOPLEFT")
			effect:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT")
			
			if newEffect.border == nil then
				texture:SetPoint("TOPLEFT", border, "TOPLEFT", 1, 1)
				texture:SetPoint("BOTTOMRIGHT", border, "BOTTOMRIGHT", -1, -1)				
			else
				texture:SetPoint("CENTER", border, "CENTER")			
				local width, height = icon:GetWidth(), icon:GetHeight()
			
				if newEffect.border ~= nil then
					width = math.floor(width - (newEffect.border * 2 * (50 / width)))
					height = math.floor(height - (newEffect.border * 2 * (50 / height)))
					
				end
				
				texture:SetWidth(width)
				texture:SetHeight(height)
			end			
			
			border:SetVisible(false)
		end
		
		effect:SetVisible(true)		
		
	end
	
	function icon:Recycle()
		icon:SetVisible(false)
		icon:EventDetach(Event.UI.Input.Mouse.Cursor.In, nil, name .. '.Mouse.Cursor.In')
		icon:EventDetach(Event.UI.Input.Mouse.Cursor.Out, nil, name .. '.Mouse.Cursor.Out')
		icon:EventDetach(Event.UI.Input.Mouse.Left.Down, nil, name .. ".Left.Down")
		icon:EventDetach(Event.UI.Input.Mouse.Cursor.Move, nil, name .. ".Cursor.Move")
		icon:EventDetach(Event.UI.Input.Mouse.Left.Up, nil, name .. ".Left.Up")
	end
	
	if privateVars.eventHandlers[name] == nil then -- if event exists the icon was used before	
		privateVars.eventHandlers[name] = {}
		privateVars.events[name] = {}
		privateVars.eventHandlers[name]["Moved"], privateVars.events[name]["Moved"] = Utility.Event.Create(addonInfo.identifier, name .. "Moved")
	end
	
	local oSetVisible = icon.SetVisible
	
	function icon:SetVisible(flag)
		
		oSetVisible(self, flag)
		if flag == false then
			if privateVars.uiElements.infoBlock ~= nil then privateVars.uiElements.infoBlock:SetVisible(false) end
		end
		
	end
	
	return icon
	
end