local addonInfo, privateVars = ...

function nkRebuff.displayOrCreateIcon (iconType, buffID, infoBlockHeader, infoBlockDesc )

	if privateVars.uiElements.icons[iconType][buffID] == nil then
		local storedIconData = nkRBIcons[nkRBConfig.activeSet][iconType][buffID]		
		if storedIconData == nil then 
			storedIconData = nkRBConfig.iconDefaults
			storedIconData.textureType = "Rift"
			storedIconData.texturePath = nkRBData[nkRBConfig.activeSet][iconType][buffID].icon
			storedIconData.label = nkRBData[nkRBConfig.activeSet][iconType][buffID].name
		end
		
		privateVars.uiElements.icons[iconType][buffID] = nkRebuff.createIcon (iconType, buffID, storedIconData)
	elseif privateVars.uiElements.icons[iconType][buffID]:GetVisible() == true then
		if infoBlockHeader ~= nil and infoBlockDesc ~= nil then	
			privateVars.uiElements.icons[iconType][buffID]:SetInfoBlock(infoBlockHeader, infoBlockDesc)
		end
			
		return privateVars.uiElements.icons[iconType][buffID]
	end
		
	privateVars.uiElements.icons[iconType][buffID]:SetTimer("")
	privateVars.uiElements.icons[iconType][buffID]:SetStack("")

	if infoBlockHeader ~= nil and infoBlockDesc ~= nil then	
		privateVars.uiElements.icons[iconType][buffID]:SetInfoBlock(infoBlockHeader, infoBlockDesc)
	end
	
	privateVars.uiElements.icons[iconType][buffID]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nkRBIcons[nkRBConfig.activeSet][iconType][buffID].xpos, nkRBIcons[nkRBConfig.activeSet][iconType][buffID].ypos)
	
	if privateVars.uiElements.icons[iconType][buffID]:GetVisible() ~= true then
		--nkRebuff.logDebug ("display icon: [" .. iconType .. "/" .. buffID .. "]")
		privateVars.uiElements.icons[iconType][buffID]:SetVisible(true)
	end
		
	privateVars.uiElements.icons[iconType][buffID]:SetAlpha(1)
	nkRebuff.storeIconCDState (iconType, buffID, nil)	
	
	if iconType ~= 'cooldownsPlayer' then
		local data = nkRBData[nkRBConfig.activeSet][iconType][buffID]
		if data ~= nil then
			if nkRebuff.IsCooldownActive(data.ability) then
				privateVars.uiElements.icons[iconType][buffID]:SetAlpha(0.2)
				nkRebuff.storeIconCDState (iconType, buffID, data.ability)
			end
		end
	end
	
	return privateVars.uiElements.icons[iconType][buffID]

end

function nkRebuff.storeIconCDState (iconType, buffID, ability)

	if privateVars.iconsOnCD[iconType] == nil then privateVars.iconsOnCD[iconType] = {} end
	privateVars.iconsOnCD[iconType][buffID] = ability

end

function nkRebuff.getIconCDState (ability)

	local retTable = {}
	
	for iconType, details in pairs (privateVars.iconsOnCD) do
		for id, abilityID in pairs (details) do
			if abilityID == ability then table.insert(retTable, { iconType = iconType, id = id}) end
		end
	end
	
	return retTable

end

function nkRebuff.displayAllIcons(dragableFlag)
	
	--nkRebuff.logDebug ("display all icons of set " .. nkRBConfig.activeSet)

	for iconType, iconList in pairs (nkRBIcons[nkRBConfig.activeSet]) do
		for id, iconDef in pairs (iconList) do
			if privateVars.uiElements.icons[iconType][id] == nil then
				nkRebuff.displayOrCreateIcon (iconType, id)
		  end
		  privateVars.uiElements.icons[iconType][id]:SetPoint("TOPLEFT", UIParent, "TOPLEFT", iconDef.xpos, iconDef.ypos)
			privateVars.uiElements.icons[iconType][id]:SetVisible(true)
			privateVars.uiElements.icons[iconType][id]:SetAlpha(1)
			privateVars.uiElements.icons[iconType][id]:SetDragable(dragableFlag)
			privateVars.uiElements.icons[iconType][id]:SetTooltip(nkRBConfig.showTooltips)
		end
	end

end

function nkRebuff.hideAllIcons()

	--nkRebuff.logDebug ("hide all icons of set " .. nkRBConfig.activeSet)
	
	for iconType, iconList in pairs (privateVars.uiElements.icons) do
		for id, iconDef in pairs (iconList) do
			privateVars.uiElements.icons[iconType][id]:SetVisible(false)
			privateVars.uiElements.icons[iconType][id]:SetDragable(false)
			privateVars.uiElements.icons[iconType][id]:SetTooltip(nkRBConfig.showTooltips)
		end
	end

end

function nkRebuff.createIcon (buffType, key, iconData)

	-- TODO: reUse of icons die nicht mehr im Gebrauch sind

	local icon = nkRebuff.icon ("nkRebuffIcon." .. buffType .. '.' .. key, privateVars.uiElements.context)	

	nkRebuff.UpdateIcon(icon, iconData)
	
	icon:SetDragable(false)	
	
	icon:SetValue('buffType', buffType)
	icon:SetValue('key', key)
	
	return icon
	
end

function nkRebuff.UpdateAllIcons()

	for iconType, iconList in pairs (nkRBIcons[nkRBConfig.activeSet]) do
		for id, iconData in pairs (iconList) do
			if privateVars.uiElements.icons[iconType][id] ~= nil then				
				local icon = privateVars.uiElements.icons[iconType][id]
				nkRebuff.UpdateIcon(icon, iconData)
			end
		end
	end

end

function nkRebuff.UpdateIcon(icon, iconData)

	icon:SetPoint("TOPLEFT", UIParent, "TOPLEFT", iconData.xpos, iconData.ypos)
	
	if iconData.useDefaults == true then
		icon:ShowLabel(nkRBConfig.iconDefaults.showLabel)
		icon:ShowBorder(nkRBConfig.iconDefaults.showBorder)
		icon:ShowTimer(nkRBConfig.iconDefaults.showTimer)
		icon:ShowStack(nkRBConfig.iconDefaults.showStack)
		icon:SetScale(nkRBConfig.iconDefaults.scale / 100)
		
		icon:SetBorderColor (nkRBConfig.iconDefaults.borderColor[1], nkRBConfig.iconDefaults.borderColor[2], nkRBConfig.iconDefaults.borderColor[3], nkRBConfig.iconDefaults.borderColor[4])
		icon:SetLabelColor (nkRBConfig.iconDefaults.labelColor[1], nkRBConfig.iconDefaults.labelColor[2], nkRBConfig.iconDefaults.labelColor[3], nkRBConfig.iconDefaults.labelColor[4])
		icon:SetTimerColor (nkRBConfig.iconDefaults.timerColor[1], nkRBConfig.iconDefaults.timerColor[2], nkRBConfig.iconDefaults.timerColor[3], nkRBConfig.iconDefaults.timerColor[4], 
		                    nkRBConfig.iconDefaults.timerOutlineColor[1],nkRBConfig.iconDefaults.timerOutlineColor[2],nkRBConfig.iconDefaults.timerOutlineColor[3])
		icon:SetStackColor (nkRBConfig.iconDefaults.stackColor[1], nkRBConfig.iconDefaults.stackColor[2], nkRBConfig.iconDefaults.stackColor[3], nkRBConfig.iconDefaults.stackColor[4])
	else
		icon:ShowLabel(iconData.showLabel)
		icon:ShowBorder(iconData.showBorder)
		icon:ShowTimer(iconData.showTimer)
		icon:ShowStack(iconData.showStack)
		icon:SetScale(iconData.scale / 100)
		
		icon:SetBorderColor (iconData.borderColor[1], iconData.borderColor[2], iconData.borderColor[3], iconData.borderColor[4])
		icon:SetLabelColor (iconData.labelColor[1], iconData.labelColor[2], iconData.labelColor[3], iconData.labelColor[4])
		icon:SetTimerColor (iconData.timerColor[1], iconData.timerColor[2], iconData.timerColor[3], iconData.timerColor[4],
		                    iconData.timerOutlineColor[1],iconData.timerOutlineColor[2],iconData.timerOutlineColor[3])
		icon:SetStackColor (iconData.stackColor[1], iconData.stackColor[2], iconData.stackColor[3], iconData.stackColor[4])
	end
	
	icon:SetTexture(iconData.textureType, iconData.texturePath)
	icon:SetLabel(nkRebuff.ComputeLabel(iconData.label))
	
	icon:SetEffect (privateVars.effects[nkRBConfig.iconDefaults.effect])

end

function nkRebuff.ComputeLabel (label)

	if label == nil then return end

	if nkRBConfig.iconDefaults.shortLabels == true then
		local tempValue
		local shortenedText = ''
		if string.find (label, ' ') == nil and string.find(label, '-') ~= nil then
			--tempValue = EnKai.strings.split(label, '-')
			tempValue = EnKai.strings.split(label, '-')
		else
			tempValue = EnKai.strings.split(label, ' ')
			--tempValue = EnKai.strings.split(label, ' ')
		end
		
		for idx2 = 1, #tempValue, 1 do
			shortenedText = shortenedText .. string.sub (tempValue[idx2], 1, 1)
		end		
		return shortenedText
	else
		return label
	end

end