local addonInfo, privateVars = ...

if nkRebuff ~= nil then return end
nkRebuff = {}

privateVars.eventHandlers = {}
privateVars.events = {}

privateVars.unitDetails = {}
privateVars.isSecure = Inspect.System.Secure()
privateVars.buffsInitialized = false

privateVars.uiElements = {}
privateVars.uiElements.icons = { buffsPlayer = {}, debuffsTarget = {}, cooldownsPlayer = {}, buffsPet = {} }

privateVars.uiElements.context = UI.CreateContext("nkRebuff")

privateVars.uiElements.configContext = UI.CreateContext ("nkRebuffConfig")
privateVars.uiElements.configContext:SetStrata ('dialog')

privateVars.uiElements.dialogContext = UI.CreateContext("nkRebuffDialog")
privateVars.uiElements.dialogContext:SetStrata ('topmost')

privateVars.uiElements.secureContext = UI.CreateContext("nkRebuffSecure")
privateVars.uiElements.secureContext:SetSecureMode('restricted')

function nkRebuff.main(_, addon)

	if addon == addonInfo.identifier then
	
		-- nkRebuff.logDebug ("nKRebuff startup (main)")		
	
		table.insert(Command.Slash.Register("nkRebuff"), {nkRebuff.commandHandler, "nkRebuff", "config"})
		table.insert(Command.Slash.Register("nkr"), {nkRebuff.commandHandler, "nkRebuff", "config"})
		table.insert(Command.Slash.Register("nkrb"), {nkRebuff.commandHandler, "nkRebuff", "config"})
		
		local items = {
		{ label = privateVars.langTexts.various.buttonMenuConfig, callBack = function () nkRebuff.ShowConfig() end},
		{ seperator = true },
		{ label = privateVars.langTexts.analyze, callBack = function ()  
			if privateVars.uiElements.Analyzer == nil then 
				privateVars.uiElements.Analyzer = nkRebuff.Analyzer() 
			elseif privateVars.uiElements.Analyzer:GetVisible() == true then
				privateVars.uiElements.Analyzer:SetVisible(false)
			else
				privateVars.uiElements.Analyzer:ClearGrid()
				privateVars.uiElements.Analyzer:SetVisible(true)
			end			
		end },
		{ seperator = true }
		}

		for idx = 1, #nkRBSets, 1 do
			table.insert (items, { label = nkRBSets[idx].name, callBack = function() nkRebuff.activateSet (idx) end })
		end
		
		if nkPanel ~= nil and nkRBConfig.nkPanel == true then
			privateVars.uiElements.panel = nkRebuff.nkPanelPlugin(items)
			nkPanel.api.registerPlugin('nkRebuff', privateVars.uiElements.panel)
			privateVars.uiElements.panel:SetTitle(nkRBSets[nkRBConfig.activeSet].name)			
		else			
			EnKai.manager.init('nkRebuff', items, nil)			
		end
		
		if nkRBConfig.buffButton ~= "0" then nkRebuff.buildReBuff() end
		
		EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)
		
		Command.Event.Attach(Event.System.Secure.Leave, nkRebuff.CheckSecureLeave, "nkRebuff.System.Secure.Leave")
		Command.Event.Attach(Event.System.Secure.Enter, nkRebuff.CheckSecureEnter, "nkRebuff.System.Secure.Enter")
		Command.Event.Attach(Event.TEMPORARY.Role, nkRebuff.RoleChange, "nkRebuff.Unit.Detail.Role")
					
		privateVars.iconsOnCD = {}
					
		-- nkRebuff.logDebug ("nKRebuff loaded (main)")
	end
	
end

function nkRebuff.commandHandler(commandline)

	if string.find(commandline, "debug") ~= nil then
		if nkRBConfig.debug == true then
			Command.Console.Display("general", true, "nkRebuff debug mode: <font color='#FF0000'>" .. "inactive" .. "</font> ", true)
			nkRBConfig.debug = false 
		else 
			Command.Console.Display("general", true, "nkRebuff debug mode: <font color='#FF0000'>" .. "active" .. "</font> ", true)
			nkRBConfig.debug = true
			nkRBDebug = ""
		end
	elseif string.find(commandline, "reset") ~= nil then
		nkRBSets = nil
		nkRBIcons = nil
		nkRBData = nil
		nkRBConfig = nil
	elseif string.find(commandline, 'int001') ~= nil then
		nkRebuff.setBuffButtonMacro(true)
	elseif string.find(commandline, 'set=') ~= nil then
		local pos = string.find(commandline, 'set=')
		local setno = tonumber(string.sub(commandline, pos+4, string.len(commandline)))
		
		if setno < 1 or setno > #nkRBSets then
			print (string.format(privateVars.langTexts.various.commandSetRefuse, #nkRBSets))
		else
			nkRebuff.activateSet (setno)
		end
	else
		nkRebuff.ShowConfig()
	end
	
end

function nkRebuff.settingsHandler(_, addon) 

	if addon == addonInfo.identifier then
	
		-- nkRebuff.logDebug ("loading settings")

		if nkRBIcons ~= nil and nkRBIcons[1] == nil then 
			-- fix for stupid set problem. can be remove in one or two versions
			nkRBIcons = nil
			nkRBData = nil
			nkRBConfig.activeSet = 1
		end
	
		if nkRBSets == nil then nkRBSets = {{ role = 0, name = 'default', buffsPlayer = {}, buffsPet = {}, debuffsTarget = {}, cooldownsPlayer = {}} } end
		if nkRBIcons == nil then nkRBIcons = {{ buffsPlayer = {}, buffsPet = {}, debuffsTarget = {}, cooldownsPlayer = {}  }} end
		if nkRBData == nil then nkRBData = {{ buffsPlayer = {}, buffsPet = {}, debuffsTarget = {}, cooldownsPlayer = {} }} end
		if nkRBConfig == nil then nkRBConfig = { showTooltips = true, buffButton = 'always', buffButtonScale = 100, buffButtonXPos = 1600, buffButtonYPos = 75, showBarsDuringConfig = true, active = true, activeSet = 1, combatOnly = false, throttle = 1, locked = true, autoRoleSwitch = false, debuffWarning = true, counterCorrection = 0,
																iconDefaults = { shortLabels = false, effect = 'gloss', useDefaults = true, warnDurationLessThan = 20, showLabel = true, showBorder = true, showTimer = true, showStack = false, 
																labelColor = {1, 1, 1, 1}, borderColor = {0, 0, 0, 1}, timerColor = {1, 1, 1, 1}, timerOutlineColor = {0, 0, 0, 1}, stackColor = {1, 1, 1, 1}, scale = 100, textureType = 'Rift', texturePath = nil, xpos = 400, ypos = 100}} end
			
		if nkRBConfig.iconDefaults.shortLabels == nil then nkRBConfig.iconDefaults.shortLabels = false end -- config addition 3.0.9
		if nkRBConfig.iconDefaults.effect == nil then nkRBConfig.iconDefaults.effect = 'gloss' end -- config addition 3.0.9
		
		-- config change 3.8.0
		
		if nkRBConfig.showTooltips == nil then
			nkRBConfig.showTooltips = true
		end
		
		-- config change 3.7.2
		
		if nkRBConfig.checkforupdates == nil then
			nkRBConfig.checkforupdates = false
		end
		
		-- config change 3.5.7
		
		if nkRBConfig.iconDefaults.xpos == 100 then
			nkRBConfig.iconDefaults.xpos = 400
		end
		
		-- config addition 3.4.0
		
		if nkRBConfig.buffButton == nil then
			nkRBConfig.buffButton = 'always'
			nkRBConfig.buffButtonScale = 100
			nkRBConfig.buffButtonXPos = 1600
			nkRBConfig.buffButtonYPos = 75
		end
				
		-- config addition 3.1.3
		
		for k, v in pairs (nkRBSets) do
			if v.buffsPet == nil then nkRBSets[k].buffsPet = {} end
		end
		
		for k, v in pairs (nkRBIcons) do
			if v.buffsPet == nil then nkRBIcons[k].buffsPet = {} end
		end
		
		for k, v in pairs (nkRBData) do
			if v.buffsPet == nil then nkRBData[k].buffsPet = {} end
		end
		
		 -- config addition 3.3.0
		
		if nkRBConfig.iconDefaults.showIfActive == nil then nkRBConfig.iconDefaults.showIfActive = false end
		
		for setNo, details in pairs (nkRBIcons) do
			for buffType, icons in pairs(details) do
				for key, _ in pairs(icons) do
					if nkRBIcons[setNo][buffType][key].showIfActive == nil then nkRBIcons[setNo][buffType][key].showIfActive = false end
				end
			end
		end
		
		-- config addition 3.9.0
		
		if nkRBConfig.nkPanel == nil then nkRBConfig.nkPanel = true end 
		
		-- config addition 3.9.2

		if type(nkRBConfig.iconDefaults.labelColor) ~= 'table' then
			local r, g, b, a = nkRebuff.translateColor (nkRBConfig.iconDefaults.labelColor)
			nkRBConfig.iconDefaults.labelColor = {r, g, b, 1} 
			
			local r, g, b, a = nkRebuff.translateColor (nkRBConfig.iconDefaults.borderColor)
			nkRBConfig.iconDefaults.borderColor = {r, g, b, 1}
			
			local r, g, b, a = nkRebuff.translateColor (nkRBConfig.iconDefaults.timerColor)
			nkRBConfig.iconDefaults.timerColor = {r, g, b, 1}
			
			local r, g, b, a = nkRebuff.translateColor (nkRBConfig.iconDefaults.stackColor)
			nkRBConfig.iconDefaults.stackColor = {r, g, b, 1}
				
			for setNo, setDetails in pairs (nkRBIcons) do
				for buffType, iconDetails in pairs (setDetails) do
					for key, icon in pairs(iconDetails) do
						local r, g, b, a = nkRebuff.translateColor (nkRBIcons[setNo][buffType][key].labelColor)
						nkRBIcons[setNo][buffType][key].labelColor = {r, g, b, 1} 
						
						local r, g, b, a = nkRebuff.translateColor (nkRBIcons[setNo][buffType][key].borderColor)
						nkRBIcons[setNo][buffType][key].borderColor = {r, g, b, 1}
						
						local r, g, b, a = nkRebuff.translateColor (nkRBIcons[setNo][buffType][key].timerColor)
						nkRBIcons[setNo][buffType][key].timerColor = {r, g, b, 1}
						
						local r, g, b, a = nkRebuff.translateColor (nkRBIcons[setNo][buffType][key].stackColor)
						nkRBIcons[setNo][buffType][key].stackColor = {r, g, b, 1}
					end
				end 				
			end 
		end	
		
		if nkRBConfig.iconDefaults.timerOutlineColor == nil then
		  nkRBConfig.iconDefaults.timerOutlineColor = {0, 0, 0, 1}
		end
		
		for setNo, setDetails in pairs (nkRBIcons) do
        for buffType, iconDetails in pairs (setDetails) do
          for key, icon in pairs(iconDetails) do
             if icon.timerOutlineColor == nil then
               nkRBIcons[setNo][buffType][key].timerOutlineColor = {1, 1, 1, 1} 
             end
          end
        end
    end
			
	end
					
end

-- sub routines

function nkRebuff.activateSet (set)
	
	if set > #nkRBSets then return end
	if privateVars.buffs.units.player == nil then return end
	
	if Inspect.System.Secure() == true then
		Command.Console.Display("general", true, privateVars.langTexts.msgActivateSetInCombat, true)
		return
	end 	
		
	-- nkRebuff.logDebug ("activate set: " .. set)
	
	nkRebuff.hideAllIcons()
	
	nkRBConfig.activeSet = set	
	
	nkRebuff.LoadCurrentBuffs(privateVars.buffs.units.player, true)
	nkRebuff.CheckMissingBuffs(false)
	nkRebuff.CheckMissingBuffsPet(false)
	nkRebuff.CheckCooldownsReady()
	
	if privateVars.uiElements.configUI ~= nil then
		privateVars.uiElements.configUI.setinfo:SetText(nkRBSets[nkRBConfig.activeSet].name)
		if privateVars.uiElements.configUI:GetVisible() == true then nkRebuff.displayAllIcons(true) end
	end

	if privateVars.uiElements.panel ~= nil then
		privateVars.uiElements.panel:SetTitle(nkRBSets[nkRBConfig.activeSet].name)
	end
	
	nkRebuff.updateReBuff()
	
	print (string.format(privateVars.langTexts.various.activateSet, nkRBSets[set].name))	
	
end

-- ==================== EVENT FUNCTIONS ====================

function nkRebuff.CheckSecureEnter ()
	
	-- nkRebuff.logDebug ("secure enter")

	if privateVars.buffs.units.playerPet ~= nil then
		local tmpTime = Inspect.Time.Real()		
		if privateVars.lastPetCheck == nil or EnKai.tools.math.round((tmpTime - privateVars.lastPetCheck), 1) > 5 then
			--print ('update pet buffs')	
			nkRebuff.LoadCurrentBuffs(privateVars.buffs.units.playerPet, false)
			privateVars.lastPetCheck = tmpTime
		end
	end
	
	nkRebuff.reInitAll ()
	
	privateVars.isSecure = true

end


function nkRebuff.CheckSecureLeave ()

	-- nkRebuff.logDebug ("secure leave")

	if nkRBConfig.combatOnly == true then nkRebuff.hideAllIcons() end
	nkRebuff.setReBuffMacro()
	privateVars.isSecure = false

end

function nkRebuff.RoleChange (_, roleID)

	if nkRBConfig.autoRoleSwitch == true then
	
		-- nkRebuff.logDebug ("role change")
	 
		for idx = 1, #nkRBSets, 1 do
			if nkRBSets[idx].role == roleID then
				nkRebuff.activateSet (idx)
				return
			end
		end
	end	

	--nkRebuff.reInitAll ()
	
end

Command.Event.Attach(Event.Addon.Load.End, nkRebuff.main, "nkRebuff.Addon.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, nkRebuff.settingsHandler, "nkRebuff.SavedVariables.Load.End")

-- ==================== BUFF BUTTON FUNCTIONS ====================

function nkRebuff.buildReBuff()

	if nkRBConfig.buffButton == '0' then return end
	
	if nkRBConfig.buffButton == 'bar' then
		if privateVars.uiElements.buffButton ~= nil then privateVars.uiElements.buffButton:SetVisible(false) end  
		
		if privateVars.uiElements.buffButtonBar == nil then 
			nkRebuff.buildBuffButtonBar()
		else
			nkRebuff.updateBuffButtonBar()
		end
	else
		if privateVars.uiElements.buffButtonBar ~= nil then privateVars.uiElements.buffButtonBar:SetVisible(false) end
		nkRebuff.buildBuffButton()
	end	

end

function nkRebuff.setReBuffMacro()

	if nkRBConfig.buffButton == '0' then return end
	
	if nkRBConfig.buffButton == 'bar' then
		nkRebuff.setBuffButtonBarMacro()
	else
		nkRebuff.setBuffButtonMacro()
	end	

end

function nkRebuff.updateReBuff()

	if nkRBConfig.buffButton == '0' then return end
	
	if nkRBConfig.buffButton == 'bar' then
		if privateVars.uiElements.buffButtonBar == nil then 
			nkRebuff.buildBuffButtonBar()
		else
			nkRebuff.updateBuffButtonBar()
		end
	else
		nkRebuff.setBuffButtonMacro()
	end	

end


function nkRebuff.buildBuffButtonBar()

	if privateVars.uiElements.buffButtonBar ~= nil then return end

	privateVars.uiElements.buffButtonBar = UI.CreateFrame('Frame', 'nkRB.buffButtonBar', privateVars.uiElements.secureContext)
	privateVars.uiElements.buffButtonBar:SetSecureMode('restricted')
	privateVars.uiElements.buffButtonBar:SetVisible(false)
	local subButtons = {}
	local subButtonIDs = {}
	local object, to = privateVars.uiElements.buffButtonBar, "TOPLEFT"
	
	local width, height = 0, 0 
	
	local types = {'buffsPlayer', 'buffsPet' }
	local units = {'player', 'player.pet'}
	
	for k, v in pairs (types) do
		if v ~= 'buffPet' or Inspect.Unit.Detail('player.pet') ~= nil then
			if nkRBSets[nkRBConfig.activeSet][v] ~= nil then					
				for id, details in pairs (nkRBSets[nkRBConfig.activeSet][v]) do
					if details.active == true then
					
						local buffData = nkRBData[nkRBConfig.activeSet][v][id]
						if buffData ~= nil and (buffData.duration == nil or buffData.duration > 120) then
						
							local button = EnKai.uiCreateFrame("nkActionButton", "nkRB.buffButton." .. (#subButtons + 1), privateVars.uiElements.buffButtonBar)
							button:SetScale(nkRBConfig.buffButtonScale/100)
							button:SetPoint("TOPLEFT", object, to)
							button:SetMacro("cast " .. buffData.name)
							button:SetValue('ability', buffData.name)
							button:SetTexture("Rift", buffData.icon)
							
							if #subButtons == 0 then
								Command.Event.Attach(EnKai.events["nkRB.buffButton.1"].Moved, function (_, x, y)
									nkRBConfig.buffButtonXPos = x
									nkRBConfig.buffButtonYPos = y
								end, "nkRB.buffButton.1.Moved")
							end
							
							width = width + button:GetWidth()
							height = button:GetHeight()
							table.insert(subButtons, button)

							subButtonIDs[id] = #subButtons
							object = button
							to = "TOPRIGHT"
						end
					end
				end
			end
		end
	end
	
	if #subButtons == 0 then
		local button = EnKai.uiCreateFrame("nkActionButton", "nkRB.buffButton.1" , privateVars.uiElements.buffButtonBar)
		button:SetScale(nkRBConfig.buffButtonScale/100)
		button:SetPoint("TOPLEFT", object, to)
		button:SetTexture("Rift", "Data/\\UI\\item_icons\\vanity_125_gloves_a.dds")
		
		if #subButtons == 0 then
			Command.Event.Attach(EnKai.events["nkRB.buffButton.1"].Moved, function (_, x, y)
				nkRBConfig.buffButtonXPos = x
				nkRBConfig.buffButtonYPos = y
			end, "nkRB.buffButton.1.Moved")
		end
		
		width = width + button:GetWidth()
		height = button:GetHeight()
		
		table.insert(subButtons, button)
	end
	
	privateVars.uiElements.buffButtonBar.subButtons = subButtons
	privateVars.uiElements.buffButtonBar.subButtonIDs = subButtonIDs
	
	privateVars.uiElements.buffButtonBar:SetWidth(width)
	privateVars.uiElements.buffButtonBar:SetHeight(height)
	
	if nkRBConfig.buffButtonXPos + privateVars.uiElements.buffButtonBar:GetWidth() > UIParent:GetWidth() then nkRBConfig.buffButtonXPos = UIParent:GetWidth() - privateVars.uiElements.buffButtonBar:GetWidth() end
	if nkRBConfig.buffButtonXPos < 0 then nkRBConfig.buffButtonXPos = 0 end
	
	if nkRBConfig.buffButtonYPos + privateVars.uiElements.buffButtonBar:GetHeight() > UIParent:GetHeight() then nkRBConfig.buffButtonYPos = UIParent:GetHeight() - privateVars.uiElements.buffButtonBar:GetHeight() end
	if nkRBConfig.buffButtonYPos < 0 then nkRBConfig.buffButtonYPos = 0 end
	
	privateVars.uiElements.buffButtonBar:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nkRBConfig.buffButtonXPos, nkRBConfig.buffButtonYPos)
	
	privateVars.uiElements.buffButtonBar:SetVisible(true)

end

function nkRebuff.updateBuffButtonBar ()

	for idx = 1, #privateVars.uiElements.buffButtonBar.subButtons, 1 do
		privateVars.uiElements.buffButtonBar.subButtons[idx]:SetAlpha(0.3)
		privateVars.uiElements.buffButtonBar.subButtons[idx]:SetVisible(true)
	end
	
	privateVars.uiElements.buffButtonBar.subButtonIDs = {}	
	local subButtonIDs = {}

	local object, to = privateVars.uiElements.buffButtonBar, "TOPLEFT"
	
	local width, height, counter = 0, 0, 1
	
	local types = {'buffsPlayer', 'buffsPet' }
	local units = {'player', 'player.pet'}
	
	for k, v in pairs (types) do
		if v ~= 'buffPet' or Inspect.Unit.Detail('player.pet') ~= nil then	
			if nkRBSets[nkRBConfig.activeSet][v] ~= nil then					
				for id, details in pairs (nkRBSets[nkRBConfig.activeSet][v]) do
					if details.active == true then
					
						local buffData = nkRBData[nkRBConfig.activeSet][v][id]
						if buffData ~= nil and (buffData.duration == nil or buffData.duration > 120) then
						
							local button = nil
						
							if privateVars.uiElements.buffButtonBar.subButtons[counter] == nil then									
								button = EnKai.uiCreateFrame("nkActionButton", "nkRB.buffButton." .. counter, privateVars.uiElements.buffButtonBar)							
								button:SetPoint("TOPLEFT", object, to)
							else
								button = privateVars.uiElements.buffButtonBar.subButtons[counter]
							end
							
							button:SetScale(nkRBConfig.buffButtonScale/100)
							button:SetMacro("cast " .. buffData.name)
							button:SetValue('ability', buffData.name)
							button:SetTexture("Rift", buffData.icon)
							button:SetVisible(true)
							
							width = width + button:GetWidth()
							height = button:GetHeight()
							
							if privateVars.uiElements.buffButtonBar.subButtons[counter] == nil then
								table.insert(privateVars.uiElements.buffButtonBar.subButtons, button)
								subButtonIDs[id] = #privateVars.uiElements.buffButtonBar.subButtons
							else
								subButtonIDs[id] = counter
							end
							
							object = button
							to = "TOPRIGHT"
							counter = counter + 1
						end
					end
				end
			end
		end
	end
	
	if counter < #privateVars.uiElements.buffButtonBar.subButtons then
		for idx = counter, #privateVars.uiElements.buffButtonBar.subButtons, 1 do
			if counter > 1 or idx > 1 then
				privateVars.uiElements.buffButtonBar.subButtons[idx]:SetVisible(false)
			end
		end
	end
	
	if counter == 1 then
		privateVars.uiElements.buffButtonBar.subButtons[1]:SetTexture("Rift", "Data/\\UI\\item_icons\\vanity_125_gloves_a.dds")
	end
	
	privateVars.uiElements.buffButtonBar.subButtonIDs = subButtonIDs

end

function nkRebuff.setBuffButtonBarMacro()

	if privateVars.uiElements.buffButtonBar == nil then return end

	if nkRBConfig.buffButton ~= 'bar' then return end
	
--	if Inspect.System.Secure() == true then return end

	for idx = 1, #privateVars.uiElements.buffButtonBar.subButtons, 1 do
		privateVars.uiElements.buffButtonBar.subButtons[idx]:SetAlpha(0.3)
	end
	
	local types = {'buffsPlayer', 'buffsPet' }
	local units = {'player', 'player.pet'}
	
	for k, v in pairs (types) do
		if v ~= 'buffPet' or Inspect.Unit.Detail('player.pet') ~= nil then	
			if nkRBSets[nkRBConfig.activeSet][v] ~= nil then					
				for id, details in pairs (nkRBSets[nkRBConfig.activeSet][v]) do
					if details.active == true then
						local index = privateVars.uiElements.buffButtonBar.subButtonIDs[id]
						if index ~= nil then
						
							if nkRebuff.IsBuffActive(units[k], id) == false then		
								local buffData = nkRBData[nkRBConfig.activeSet][v][id]
								if buffData ~= nil and (buffData.duration == nil or buffData.duration > 120) then
									privateVars.uiElements.buffButtonBar.subButtons[index]:SetAlpha(1)								
								end
							else
								privateVars.uiElements.buffButtonBar.subButtons[index]:SetAlpha(0.3)
							end
						end
					end
				end
			end
		end
	end
end

function nkRebuff.buildBuffButton()
	
	if privateVars.uiElements.buffButton ~= nil then return end
	
	-- nkRebuff.logDebug ("build buff button")				
	
	privateVars.uiElements.buffButton = EnKai.uiCreateFrame("nkActionButton", "nkRB_buffButton", privateVars.uiElements.secureContext)			
	privateVars.uiElements.buffButton:SetTexture("Rift", "Data/\\UI\\item_icons\\vanity_125_gloves_a.dds")
	privateVars.uiElements.buffButton:SetScale(nkRBConfig.buffButtonScale/100)
	
	if nkRBConfig.buffButtonXPos + privateVars.uiElements.buffButton:GetWidth() > UIParent:GetWidth() then nkRBConfig.buffButtonXPos = UIParent:GetWidth() - privateVars.uiElements.buffButton:GetWidth() end
	if nkRBConfig.buffButtonXPos < 0 then nkRBConfig.buffButtonXPos = 0 end
	
	if nkRBConfig.buffButtonYPos + privateVars.uiElements.buffButton:GetHeight() > UIParent:GetHeight() then nkRBConfig.buffButtonYPos = UIParent:GetHeight() - privateVars.uiElements.buffButton:GetHeight() end
	if nkRBConfig.buffButtonYPos < 0 then nkRBConfig.buffButtonYPos = 0 end
	
	privateVars.uiElements.buffButton:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nkRBConfig.buffButtonXPos, nkRBConfig.buffButtonYPos)
	
	privateVars.uiElements.buffButton:SetVisible(true)
			
	privateVars.uiElements.buffButton:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
		if privateVars.uiElements.infoBlock == nil then privateVars.uiElements.infoBlock = nkRebuff.infoBlock() end
		
		privateVars.uiElements.infoBlock:SetHeader(privateVars.langTexts.buffButtonHintHeader)
		privateVars.uiElements.infoBlock:SetDescription(string.format(privateVars.langTexts.buffButtonHintDesc, privateVars.uiElements.buffButton:GetValue('ability')))
		
		local width, height = privateVars.uiElements.infoBlock:GetWidth(), privateVars.uiElements.infoBlock:GetHeight() 
		privateVars.uiElements.infoBlock:ClearAll()
		privateVars.uiElements.infoBlock:SetWidth(width)
		privateVars.uiElements.infoBlock:SetHeight(height)
		privateVars.uiElements.infoBlock:SetPoint("BOTTOMRIGHT", privateVars.uiElements.buffButton, "TOPRIGHT")
		privateVars.uiElements.infoBlock:SetVisible(true)			
	end, "nkRB_buffButton" .. '.Mouse.Cursor.In')
	
	privateVars.uiElements.buffButton:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
		if privateVars.uiElements.infoBlock ~= nil then privateVars.uiElements.infoBlock:SetVisible(false) end
	end, "nkRB_buffButton" .. '.Mouse.Cursor.Out')
	
	Command.Event.Attach(EnKai.events.nkRB_buffButton.Moved, function (_, x, y)
	
		if Inspect.System.Secure() == true then return end
	
		nkRBConfig.buffButtonXPos = x
		nkRBConfig.buffButtonYPos = y
		
		if nkRBConfig.buffButtonXPos + privateVars.uiElements.buffButton:GetWidth() > UIParent:GetWidth() then nkRBConfig.buffButtonXPos = UIParent:GetWidth() - privateVars.uiElements.buffButton:GetWidth() end
		if nkRBConfig.buffButtonXPos < 0 then nkRBConfig.buffButtonXPos = 0 end
		
		if nkRBConfig.buffButtonYPos + privateVars.uiElements.buffButton:GetHeight() > UIParent:GetHeight() then nkRBConfig.buffButtonYPos = UIParent:GetHeight() - privateVars.uiElements.buffButton:GetHeight() end
		if nkRBConfig.buffButtonYPos < 0 then nkRBConfig.buffButtonYPos = 0 end
		
		privateVars.uiElements.buffButton:SetPoint("TOPLEFT", UIParent, "TOPLEFT", nkRBConfig.buffButtonXPos, nkRBConfig.buffButtonYPos)
	end, "nkRB_buffButton.Moved")
	
end

function nkRebuff.setBuffButtonMacro(ignoreFirst)

	if privateVars.uiElements.buffButton == nil then return end

	if nkRBConfig.buffButton == '0' or nkRBConfig.buffButton == 'all' then return end
	if Inspect.System.Secure() == true then return end
	
	-- nkRebuff.logDebug ("set buff button macro")
	
	local firstIgnored = false
	
	local types = {'buffsPlayer', 'buffsPet' }
	local units = {'player', 'player.pet'}
	
	for k, v in pairs (types) do
		if v ~= 'buffPet' or Inspect.Unit.Detail('player.pet') ~= nil then	
			if nkRBSets[nkRBConfig.activeSet][v] ~= nil then					
				for id, details in pairs (nkRBSets[nkRBConfig.activeSet][v]) do
					if details.active == true then
						if nkRebuff.IsBuffActive(units[k], id) == false then		
							local buffData = nkRBData[nkRBConfig.activeSet][v][id]
							if buffData ~= nil and buffData.name ~= nil and (buffData.duration == nil or buffData.duration > 120) then
								if ignoreFirst ~= true or firstIgnored == true then
									privateVars.uiElements.buffButton:SetMacro("cast " .. buffData.name .. '\nnkRB int001')
									privateVars.uiElements.buffButton:SetValue('ability', buffData.name)
									privateVars.uiElements.buffButton:SetActiveState(false)
									privateVars.uiElements.buffButton:SetVisible(true)
									return
								elseif ignoreFirst == true then
									firstIgnored = true
								end
							end
						end
					end
				end
			end
		end
	end
	
	privateVars.uiElements.buffButton:SetMacro(nil)
	privateVars.uiElements.buffButton:SetActiveState(true)
	
	if privateVars.uiElements.infoBlock ~= nil then privateVars.uiElements.infoBlock:SetVisible(false) end
	
	if nkRBConfig.buffButton == 'rebuff' then privateVars.uiElements.buffButton:SetVisible(false) end

end

-- ==================== HELPER FUNCTIONS ====================

function nkRebuff.ShowConfig()

	if privateVars.uiElements.configUI == nil then 
		privateVars.uiElements.configUI = nkRebuff.Config()
	elseif privateVars.uiElements.configUI:GetVisible() == true then
		privateVars.uiElements.configUI:SetVisible(false)
	else
		privateVars.uiElements.configUI:SetVisible(true) 
	end

end

function nkRebuff.reInitAll ()

	--nkRebuff.logDebug ("reInitAll")
	
	if privateVars.buffsInitialized == false then
		nkRebuff.LoadCurrentBuffs(privateVars.buffs.units.player, true)
		privateVars.buffsInitialized = true
	end
	
	nkRebuff.CheckMissingBuffs(true)
	nkRebuff.CheckMissingBuffsPet(true)
	nkRebuff.CheckMissingDebuffs(true)
	nkRebuff.CheckCooldownsReady()

	nkRebuff.setReBuffMacro()
	
end

function nkRebuff.getUnitDetails (unit)

	-- nkRebuff.logDebug ("getUnitDetails for " .. unit)

	if privateVars.unitDetails[unit] == nil then
		privateVars.unitDetails[unit] = Inspect.Unit.Detail(unit)
	end
	
	return privateVars.unitDetails[unit]

end


function nkRebuff.translateColor (color)

	if color == '000000' or color == 'black' then
		return 0, 0, 0, 1
	elseif color == '7F0000' or color == 'red' then
		return 0.498, 0, 0, 1
	elseif color == 'FFD800' or color == 'yellow' then
		return 1, 0.847, 0, 1
	elseif color == '0094FF' or color == 'blue' then
		return 0, 0.580, 1, 1
	else
		return 1, 1, 1 ,1
	end

end

function nkRebuff.confirmDialog (message, yesFunc, noFunc)

	if privateVars.uiElements.confirmDialog == nil then
		privateVars.uiElements.confirmDialog = EnKai.uiCreateFrame("nkDialog", "nkRW.confirmDialog", privateVars.uiElements.dialogContext)
		privateVars.uiElements.confirmDialog:SetWidth(500)
		privateVars.uiElements.confirmDialog:SetHeight(250)
		privateVars.uiElements.confirmDialog:SetType('confirm')
	end
	
	privateVars.uiElements.confirmDialog:SetMessage(message)
	privateVars.uiElements.confirmDialog:SetVisible(true)
	
	Command.Event.Detach(EnKai.events['nkRW.confirmDialog'].LeftButtonClicked, nil, "nkRW.confirmDialog.LeftButtonClicked") -- delete possible earlier event handler and replace with new one
	Command.Event.Detach(EnKai.events['nkRW.confirmDialog'].RightButtonClicked, nil, "nkRW.confirmDialog.RightButtonClicked") -- delete possible earlier event handler and replace with new one
		
	Command.Event.Attach(EnKai.events['nkRW.confirmDialog'].LeftButtonClicked, function ()
		yesFunc()
	end, "nkRW.confirmDialog.LeftButtonClicked")
	
	if noFunc ~= nil then	
		Command.Event.Attach(EnKai.events['nkRW.confirmDialog'].RightButtonClicked, function ()
			noFunc()
		end, "nkRW.confirmDialog.RightButtonClicked")
	end
		
end

function nkRebuff.messageDialog (message)

	if privateVars.uiElements.messageDialog == nil then
		privateVars.uiElements.messageDialog = EnKai.uiCreateFrame("nkDialog", "nkRW.messageDialog", privateVars.uiElements.dialogContext)
		privateVars.uiElements.messageDialog:SetWidth(500)
		privateVars.uiElements.messageDialog:SetHeight(200)
		privateVars.uiElements.messageDialog:SetType('message')
	end
	
	privateVars.uiElements.messageDialog:SetMessage(message)
	privateVars.uiElements.messageDialog:SetVisible(true)

end

function nkRebuff.printError(errTypeLabel, errMessage)

	Command.Console.Display("general", true, "<font color='#FF0000'>" .. errTypeLabel .. "</font> ", true)
	Command.Console.Display("general", true, "<font color='#FFFFFF'>" .. errMessage .. "</font> ", true)
		
end

function nkRebuff.logDebug (message)

	if nkRBConfig == nil then return end
	if nkRBConfig.debug ~= true then return end
	nkRBDebug = nkRBDebug .. os.date("%H:%M:%S", os.time()) .. ": " .. message .. "\n";

end