local addonInfo, privateVars = ...

privateVars.manual = {
		{ parent = nil, title = "Introduction", content = {{ type = "text", text = "nkRebuff is a Rift addon which reminds you of important things concerning buffs, debuffs and cooldowns. It will tell you if you or your pet are missing an important buff or if your target is missing an important debuff. Also it will tell you about abilities whose cooldowns are ready to use.\n\nAll this requires some very simple steps to set up. A basic configuration for your char can be done in a few minutes. It also offers you a lot of options to customize the addon to your needs.\n\nThis document aims at giving a basic understanding of how to use the addon. On top of this we'll go through some of the more advanced features. Please be aware that addons are constantly updated with new functionality or require change due to a Rift patch. So this documentation might not always be covering the latest version available."}} },
		{ parent = nil, title = "The basics" },
		{ parent = "The basics", title = "Accessing the addon", content = {	{ type = "text", text = "When you first install the addon a button will appear next to the minimap. If you moved your minimap to a different location please look in the top right area of the screen. The button can be easily identified.\n\nBy left-clicking on the button a menu will open up.\n\n"},																
																			{ type = "image", textureType = "nkRebuff", texturePath = "manual/minimapButton.png", width = 92, height = 90, align = "lastright" },
																			{ type = "text", text = "If nkRebuff is the only addon of 'nk' Range installed, there'll be just one entry 'nkRebuff'. If you installed others of my addons the corresponding addon names will be shown there as well. For now we are only interested in the nkRebuff entry. By clicking on that entry a sub menu with three options will show up:\n\n" },
																			{ type = "text", text = "- Configuration\n- Analyse\n- Manual\n- default (If just installed)\n\n"},
																			{ type = "image", textureType = "nkRebuff", texturePath = "manual/minimapMenu.png", width = 172, height = 90, align = "lastright" },
																			{ type = "text", text = "By clicking on the <font color='#0094FF'>Configuration</font> entry the addon will open up and a new window will appear on screen.\n\nThe other options are covered in more details in the sections <u>Analysing your abilities</u> and <u>Managing abilities</u>.\n\nInstead of using the minimap button you could also type <font color='#0094FF'>/nkrb</font> in the chat to open the configuration."}
																			}},
		{ parent = "The basics", title = "The configuration window", content = {{ type = "text", text = "nkRebuff's configuration consists of five different sections which are represented by the four tabs at the top of the window.\n\nYou can switch the tab by simply clicking on it."},
																				{ type = "image", textureType = "nkRebuff", texturePath = "manual/configuration.png", width = 250, height = 184, align = "lastright"},
																				{ type = "header", text = "Settings section"},
																				{ type = "text", text = "In this section you can specify a number of generic settings which apply globally. Also you can manage the nkRebuff sets in here. Please see chapters <u>How does nkRebuff know what I want to track?</u> and <u>Managing abilities</u> for more information about this section."},
																				{ type = "header", text = "Icon defaults"},
																				{ type = "text", text = "An icon is a small graphical on-screen thingy which is there to remind you something important. What is important is up to you and how you set up nkRebuff. Generally a text label and icon will be displayed whenever you need to renew a buff, apply a debuff or a cooldown is ready.\n\nThe icon defaults section is of course all about those icons. See chapter <u>Change the default look of the icons</u> for detailed information of the settings and their effects."},
																				{ type = "header", text = "Abilities"},
																				{ type = "text", text = "Now this section is where all the fun is. In this section you'll find all the buffs, debuffs and cooldowns you want to track and their specific icon settings. Please see chapter <u>Abilities and their onscreen representation</u> for what you can and should do in this section."},
																				{ type = "header", text = "Import/Export"},
																				{ type = "text", text = "Using this tab you can import and export nkRebuff configuration sets so that they are easily shareable among the community."},
																				{ type = "header", text = "About nkRebuff"},
																				{ type = "text", text = "This is my small place to talk to you guys out there. You'll find some information about the addon and me."}
																				}},
		{ parent = "The basics", title = "What is an icon?", content = {{ type = "text", text = "An icon is a small graphical on-screen thingy which is there to remind you something important. What is important is up to you and how you set up nkRebuff. Generally spoken and icon will be displayed whenever you need to renew a buff, apply a debuff or a cooldown is ready.\n\n"},
																		{ type = "image", textureType = "nkRebuff", texturePath = "manual/iconExample.png", width = 114, height = 69, align = "lastright"},
																		{ type = "text", text = "Normally the icon shown is the same as the ability used to apply that buff, debuff or to activate the ability whose cooldown you want to track. You can of course change the icon to something else in the configuration but I guess you understand what I mean.\n\nAn icon features a timer counter. For buffs and debuffs the counter will show you how many seconds are left before the buff or debuff runs out. For cooldowns the counter timer tells you how many seconds are left until the ability is ready again. By default the timer will start to show 20 seconds before it runs out, but that can be changed in the configuration globally and for each icon individually.\n\nIn addition to the timer counter an icon will show you stack information if the buff or debuff includes stacks. This is mostly useful if you reverse the functionality of nkRebuff and actually display buffs and debuffs if they are active. See chapter <u>Reversed buff and debuff mode</u> for more information about this feature of nkRebuff."}}},
		{ parent = "The basics", title = "How does nkRebuff know what to track?", content = {{ type = "text", text = "The answer is quite simple: <font color='#FF0000'>It doesn’t until you teach it</font>. In order for you not having to type in buff and ability names the addon features a function called <font color='#0094FF'>‘Analyser’</font>. You can access the Analyser either from the configuration window by clicking on the button named Analyser or by opening the nkRebuff menu on the minimap button and choosing the entry Analyser.\n\nWithin the analyser the addon automatically detects new buffs, debuffs and cooldowns being used and you can then import selected to your set configuration. How this works is described in detail in the sections <u>Analyzing your abilities</u> and <u>Managing abilities</u>." }}},
		{ parent = nil, title = "Configuring the addon", content = {{ type = "text", text = "Now that you understand how to open the addon and what the configuration window is structured like it's time to talk about how to configure the addon.\n\nPlease note that if you have configured a set the addon will always show all icons of that set when opening the configuration. This is done so that you can move around the icons to where you want to place them. Also the ReRebuff button (or bar) is automatically shown so that you can drag that button to wherever you want to place it."}}},
		{ parent = "Configuring the addon", title = "The global settings", content = {{ type = "text", text = "As mentioned above in the first tab called <font color='#0094FF'>settings</font> you'll find a couple of global settings which will affect the addon as a whole. We'll not talk about set handling here as this is covered in detail in the chapter <u>Managing abilities</u>. Instead we'll focus on the left side of settings which are structured in three sub sections."}}},
		{ parent = "The global settings", title = "Basic settings", content = {	{ type = "header", text = "Addon activated"},
																				{ type = "text", text = "The first setting is more or less self-explaining. 'Addon activated' does what is says. If the checkbox is ticked the addon will work. If not ... well why you would want to do that anyway. Please note that if you deactivate the addon with the settings it will still load. Otherwise you couldn't change your decision when you noticed that you can't live without nkRebuff."},																				
																				{ type = "image", textureType = "nkRebuff", texturePath = "manual/globalSettings.png", width = 250, height = 211, align = "lastright"},
																				{ type = "text", text = "without nkRebuff anymore. However if the addon is deactivated it will save some computing time as it will no longer process any buff, debuff and cooldown information anymore. As nkRebuff is extremely optimized the impact on CPU usage isn't much different."},
																				{ type = "header", text = "Show only in combat"},
																				{ type = "text", text = "Normally the addon would always tell you about configured things like a buff missing as soon as it happens – meaning the corresponding icon will show up. However if you're like me and love a clean UI you might not want to run around all the time with your screen full of icons telling you this and that cooldown is ready.\n\nBy checking this setting's checkbox the addon will only start showing icons and telling you to apply that much needed debuff on your target if you're actually fighting."},
																				{ type = "header", text = "Recognize role switch"},
																				{ type = "text", text = "This setting will activate the role switch routine. Normally you'll have a least one set of nkRebuff configuration (See chapter <u>Managing abilities</u> for more information on configuration sets) per role you bought. At the time of writing you can have 20 different roles for each character. If you switch a role say from melee to ranged DPS spec or from tank to healer spec (isn't Rift's soul system awesome) you will also want to switch the nkRebuff configuration set.\n\nYou could switch the set manually in the configuration. You could also switch the active set using the command line options. But wouldn't it be much smarter for nkRebuff to automatically load the correct set you'd need for that role? nkRebuff can do that and the section <u>Managing abilities</u> tells you how.\n\nIn order for nkRebuff to start checking for role changes you will need to activate this checkbox.\n\nHowever sometimes you want to change the nkRebuff set manually for different reasons. For example if you use nkWardrobe that addon can automatically switch the nkRebuff set along the gear. Others might want to use a macro which switches the role and does other stuff as well. Under these conditions it would be a waste of resources to check for role changes and you better deactivate the checkbox."},
																				{ type = "header", text = "Check for updates"},
																				{ type = "text", text = "nkRebuff uses IDs of the API to identify buffs, debuffs and cooldowns. Sometimes these get changed due to the abilities changing in a class patch. So after a Rift patch there's the small change that some of your buffs, debuffs or cooldowns don't work anymore.\n\nThe addon can identify most of the updates itself. If you activate this setting the addon will check for new versions of configured items. If a new versoin is found the addon will automatically update it's configuration and tell you so in the chat pane."},
																				{ type = "header", text = "Player debuff warning"},
																				{ type = "text", text = "Warning about debuffs on the player is a feature which was implemented in the old V2 of nkRebuff. At the time of writing this feature has not yet been implemented into V3. Therefore this setting is shown but cannot be changed so far."}
																				}},
		{ parent = "The global settings", title = "Performance", content = {{ type = "text", text = "The performance of nkRebuff V3 is awesome. It's way more efficient than the old version and most of the time it will not even show up in the CPU usage overview. Still in order to achieve the maximum possible performance some calculations are by default only done once per second.\n\nThe most notable thing is the timer and the stack counter. These are updated in the interval which is specified by this setting.\n\nSometimes you'll notice that the timer might be slightly off versus the standard buff display or other addons (for example unit frame addons). This is because nkRebuff does some rounding. Instead of just cutting of the digits it rounds the value.\n\nIf you lower the performance setting the addon will check the counter and stack much more often and therefore display more accurate values. It's up to you if you want to sacrifice a little bit of performance for a more precise timer. The impact on performance isn't huge though if you set the setting to its lowest value."}}},
		{ parent = "The global settings", title = "ReRebuff Button", content = 	{{ type = "text", text = "In chapter <u>Easy rebuffing with the Rebuff button</u> you'll find a description of what that button really does. This settings section lets you modify the look of the button."},
																				{ type = "header", text = "Show button"},
																				{ type = "text", text = "This option allows you to specify if you want a single button or a bar. If you would like to use the bar please select the corresponding option. If no you can specify if the ReRebuff button should be always shown (default) or only if something needs to be rebuffed. You can also deactivate the button (and bar) by choosing the corresponding setting in the selection box."},
																				{ type = "header", text = "Scale"},
																				{ type = "text", text = "The ReRebuff button /bar can be scaled to a better blend into your ui layout. Simply set this setting to the desired value and the ReRebuff button / bar will change directly."}}},
		{ parent = "Configuring the addon", title = "Change the default look of the icons", content = {	{ type = "header", text = "Design" },
																										{ type = "text", text = "These settings all affect how icons are being displayed - unless you override the defaults for an icon. You can choose to display either of the following icon features:\n\n- label\n- border\n- timer\n- stack\n\n"},																										
																										{ type = "image", textureType = "nkRebuff", texturePath = "manual/iconDefaults.png", width = 250, height = 199, align = "lastright"},
																										{ type = "text", text = "By unchecking the checkbox this feature will be disabled.\n\nAllthough it's an option <font color='#FF0000'>I advise against deactivating the display of counters</font>.\n\nAlso for each of the four features you can choose out of a number of predefined colors. Of course these only have any effect if the feature is activated.\n\nThe scale option is quite self-explanatory. Using this slider you can set the default size of icons."},
																										{ type = "header", text = "Functionality" },
																										{ type = "text", text = "Unless you deactivate the counter feature (Don't do it!) the addon will check if a buff or debuff is about to run out. If so it will start displaying the icon and a counter on top of the icon going from N to zero.\n\nUsing the 'n sec warning' option you can defined, how much time has to be left before the icon and counter will be shown."},
																										{ type = "header", text = "Storing your changes"},
																										{ type = "text", text = "If you do changes in this part of the configuration you will have to <font color='#FF0000'>click on the update button</font> for the changes to be stored."}																										
																									}},
		{ parent = "Configuring the addon", title = "Global icon settings", content = {	{ type = "text", text = "These settings affect all icons globally and cannot be overridden in the individual icon configuration.\n\n" },					
																						{ type = "image", textureType = "nkRebuff", texturePath = "manual/globalIconSettings.png", width = 250, height = 66, align = "lastright"},
																						{ type = "header", text = "Short labels"},
																						{ type = "text", text = "If this is activated all labels will be shortened to their first letter of each word. For example Empowering Strike will result is ES. This option is only useful if you actually display labels. In case you deactivated the global option to display labels, the shortening will of course not happen." },
																						{ type = "header", text = "Icon effect"},
																						{ type = "text", text = "There are a number of styles how nkRebuff can display the icons. Using this option you can choose one of the designs.\n\n" },
																						{ type = "image", textureType = "nkRebuff", texturePath = "manual/iconEffects.png", width = 310, height = 50, align = "left"},
																						{ type = "text", text = "\nIn order of appearance:\n\n- deactivated (no effect)\n- Basic\n- Chrome\n- Glossy\n-Round\n-Rounded corners" }
																						}},																					
		{ parent = nil, title = "Managing abilities" },
		{ parent = "Managing abilities", title = "The ability list", content = {{ type = "text", text = "The abilities tab of the configuration is where you can manage your abilities and define the way they are shown on the screen. If you open this tab you’ll find a list on the left. This list will be empty if you haven't done any analysing yet.\n\nIt is important to note that <font color='#FF0000'>the abilities tab is always linked to the activated set</font>. What set is active can easily be checked on the first tab in the set handling section or to the top right of the configuration.\n\nUsing the selection box above the list you can choose which abilities to list:\n\n"},
																				{ type = "text", text = "- Buffs on you\n- Buffs on your pet\n- Debuffs on your target\n- Cooldowns\n\nSelect one of the entries and the addon will display the corresponding abilities you identified and stored using the analyser for the active set.\n\nFor each of the entries there’s a red or green icon being shown. Red means this ability is not to be tracked by nkRebuff so anything you’ll"},
																				{ type = "image", textureType = "nkRebuff", texturePath = "manual/abilityList.png", width = 250, height = 284, align = "lastright"},
																				{ type = "text", text = "do with that ability will have no effect. Green means you want nkRebuff to track that ability and act on it.\n\nYou can change the status <font color='#FF0000'>by right clicking an entry</font> and the icon will change colour."}																				
																				} },
		{ parent = "Managing abilities", title = "Creating icons", content = {	{ type = "text", text = "To the right of the ability list there’s an area to create and modify icons. If you select an entry in the ability list by left-click it you’ll see that the right side of the configuration section will populate.\n\nIf you do not specify otherwise an icon uses the settings defined in the icon defaults tab of the configuration. See below what options are available for customizations.\n\n"},																				
																				{ type = "image", textureType = "nkRebuff", texturePath = "manual/iconDesigner.png", width = 250, height = 279, align = "lastright"},
																				{ type = "text", text = "It is very important to note that selecting an entry in the list <font color='#FF0000'>is not sufficient to create an icon</font>. You’ll have to click on the <font color='#0094FF'>Create</font> button in the icon definition section. This will finally create an icon and show it on the screen. The default location for new icons is to the left top of the screen.\n\nAs soon as an icon is created it will be stored from now on.\n\nYou’ll find that the create button will be replaced by a delete button. If you ever want to remove an icon cause you no longer want to track that ability you can do so with the delete button. This remove any reference to the icon settings so that you’ll have to re-create it later if needed.\n\nIf the same ability is used in different sets you’ll have to create an icon for the ability in each set separately."},
																			}},
		{ parent = "Managing abilities", title = "Customizing icons", content = {{ type = "text", text = "You can customize your icons both in look and behaviour. Each newly created icon uses the default icon settings (See chapter <u>Icon defaults</u>) but by unchecking the checkbox <font color='#0094FF'>use defaults</font> you can choose to override the default settings."}}},
		{ parent = "Customizing icons", title = "Changing the look", content = {{ type = "text", text = "It is possible to override the default icon settings and have a specific design for each icon. If you want to customize an icon you'll have to uncheck the checkbox <font color='#0094FF'>use defaults</font>.\n\n"},
																				{ type = "image", textureType = "nkRebuff", texturePath = "manual/iconDesignOverride.png", width = 250, height = 105, align = "lastright"},
																				{ type = "text", text = "As soon as you do this the individual options will become editable. These are the exact same features like described in chapter <u>Change the default look of the icons</u>. So please see there for a description of the features."}																				
																				}},
		{ parent = "Customizing icons", title = "Changing the behaviour", content = {	{ type = "header", text = "Show if active"},
																						{ type = "text", text = "This option activates the <font color='#0094FF'>reverse mode</font>. Please see chapter <u>Reversed buff and debuff mode</u> for more information on this."},
																						{ type = "header", text = "Runout warning"},
																					 	{ type = "text", text = "If you override the defaults, you can specify a different warning time for an icon. This can be very useful for buffs and debuffs with a rather short duration"}}},
		{ parent = "Customizing icons", title = "Choosing a different icon and label", content = {	{ type = "header", text = "Changing the icon"},
																									{ type = "text", text = "By default an icon will use the default system icon. However you can override that icon by choosing a different one in the icon setup. The addon will automatically read all your items and abilities and you can choose from these icons. If you would choose the icon of an item in your"},
																								 	{ type = "image", textureType = "nkRebuff", texturePath = "manual/imageGallery.png", width = 148, height = 99, align = "lastright"},
																								 	{ type = "text", text = " inventory you don’t need to carry that item all the time. Once an image has been set it can be displayed without the originating item being available.\n\nUsing the two arrow buttons you go through the list of icons. The centre icon is the one which is selected."},
																								 	{ type = "header", text = "Changing the label"},
																								 	{ type = "text", text = "If you do not specify otherwise in the default icon settings or individually for an icon the addon will display the name of the ability below the icon. In case you’d want to display a different label you can do so by entering a new name in the edit box. Hit <font color='#0094FF'>ENTER</font> to store the new name or <font color='#0094FF'>ESCAPE</font> to cancel the name editing.\n\nAfter finishing all your changes you have to click on the update button for the icon definition to update."},
																								 	{ type = "header", text = "Storing your changes"},
																								 	{ type = "text", text = "If you do changes in this part of the configuration you will have to <font color='#FF0000'>click on the update button</font> for the changes to be stored."}
																								 	
																								 }},
		{ parent = "Managing abilities", title = "Moving icons on the screen", content = {{ type = "text", text = "Moving icons around the screen is very easy. Simply Drag & Drop the icon to wherever on the screen you want it to be displayed.\n\n"},
																						  { type = "text", text = "<font color='#FF0000'>There’s a snapping function in place.</font> If you drag an icon near another one it will automatically position it next to it depending on where you moved it. If you moved it atop the other icon it will be positioned above it. Left, right and below works just the same. Drag it away if you want to position it somewhere else.\n\n"},
																						  { type = "image", textureType = "nkRebuff", texturePath = "manual/iconSnap.png", width = 105, height = 104, align = "lastright"},
																						  { type = "text", text = "In addition to the manual drag & drop moving you can exactly position the icon using the icon setup. There’ll you find an arrow configuration along with two edit"},
																						  { type = "image", textureType = "nkRebuff", texturePath = "manual/iconPos.png", width = 175, height = 71, align = "lastright"},
																						  { type = "text", text = "boxes for x and y coordinates. Using the arrows or by entering an exact coordinate you can position the icon.\n\nAfter finishing all your changes you have to click on the update button for the icon definition to update."}}},
		{ parent = nil, title = "Set handling"},
		{ parent = "Set handling", title = "What is a set?", content = {{ type = "text", text = "The configuration sets are a very important and flexible aspect of nkRebuff.  In simple terms a set is a collection of buffs, debuffs and cooldowns to handle (e.g. notify you) and (optionally) their individual look and behaviour.\n\nUsing the Analyser you choose which things you want to track. For each of these you then can create an icon in the configuration. For each of these icons you can then decide to use the default look or customize it. All this is eventually stored into a single configuration set.\n\n"},
																		{ type = "text", text = "In most scenarios you’d want to use a different set configuration per Rift role build you use. With Rift allowing up to 20 soul builds you can have equally many sets."},
																		{ type = "image", textureType = "nkRebuff", texturePath = "manual/sets.png", width = 250, height = 93, align = "lastright"},
																		{ type = "text", text = "You could also have multiple nkRebuff configuration sets for one soul build or use the same set for multiple soul builds. How you use the sets is up to you.\n\nWhat I do not advise is to setup mixed sets which consist of abilities of different soul builds. This would lead to icons being shown as the corresponding buffs cannot be cast as you’re lacking the ability with your build."}
																		}},
		{ parent = "Set handling", title = "Creating a new set", content = {{ type = "text", text = "Creating a new set is very easy. Simply open the configuration and click on the button <font color='#0094FF'>New set</font>. The addon will then create a new set with then name <font color='#0094FF'>New Set</font> and the number of the new set.\n\nIt will also automatically activate that new set so that you can start the Analyser and fill the set with abilities.\n\nOn the settings tab in the configuration you’ll find a selection box with all sets you created. After creating a new set that selection box will automatically be set to the new set.\n"},
																			{ type = "image", textureType = "nkRebuff", texturePath = "manual/setNameEdit.png", width = 372, height = 25, align = "left"},																			
																			{ type = "text", text = "By clicking on the pen right to the selection box you can change into “name change mode”. The selection box will be hidden and an edit box be displayed instead. In that edit box you can change the name of the set. Simply hit <font color='#0094FF'>ENTER</font> once you’re happy with the new name. If you want to leave the field without editing the name simply hit <font color='#0094FF'>ESCAPE</font>."}}},
		{ parent = "Set handling", title = "Linking a set to a role (build)", content = {	{ type = "text", text = "If you would like a set to automatically be loaded if you switch to a specific role, you’ll need to activate the global function (Checkbox Recognize role switch in the basic settings – See chapter <u>Basic settings</u>).\n\nIn addition you’d need to specify which role this set is linked to. This is done using the selection box <font color='#0094FF'>assigned role</font> below the set selection.\n\n"},
																							{ type = "image", textureType = "nkRebuff", texturePath = "manual/assignedRole.png", width = 353, height = 26, align = "left"},
																							{ type = "text", text = "\nThe roles (soul builds) are numbered 1-20. Choose the role and the addon will automatically switch to that nkRebuff set whenever you activate that role (Either by /role n, a macro button or another addon like nkWardrobe).\n\nBy choosing <font color='#0094FF'>deactivated</font> you can disable the role linking for a set. This is useful for sub-sets which you’d only use in specific situations. For example special boss encounters in raids or something like this."}}},
		{ parent = "Set handling", title = "Activating a set", content = {{ type = "text", text = "You can activate a set by a number of ways:\n\nMinimap button:\n\nThe minimap button menu will list all configured sets and you can activate a set by clicking on it.\n\nCommand line:\n\nThe following command will switch the activate set:\n\n/nkrb set=X\n\nWhere X is the number of the set. For example /nkrb set=5.\n\nIn the configuration:\n\nThe least common way to switch sets is to use the set selection box in the configuration’s settings pane. If you select a set name there the addon will automatically switch the set so that you can configure it. This however also means that if you select a set in that selection box and close the configuration that set would still be active."}}},
		{ parent = "Set handling", title = "Copying and deleting an existing set", content = {{ type = "text", text = "Using the two buttons <font color='#0094FF'>Copy Set</font> or <font color='#0094FF'>Delete Set</font> you can copy or delete the active set. In the case of copying this will create an exact copy of the active set. So if you want to copy a different than the active set you’d have to select it in the configuration’s selection box first. The same is of course true for the deletion process.\n\nA copied set get the name of the original set with the prefix <font color='#0094FF'>copy of</font>.\n\nIf you delete a set you’ll be presented by a confirmation dialog. Click on ‘Yes’ and the set is deleted. <font color='#FF0000'>There is no way to recover a deleted set!</font> Even a /reloadui will not recover the set. So make sure you pay attention to what you want to delete."}}},
		{ parent = nil, title = "Analyzing your abilities"},
		{ parent = "Analyzing your abilities", title = "First step: open the Analyzer", content = {{ type = "text", text = "As already mentioned in the basics, you’ll need to teach nkRebuff what abilities you want to track. An important thing to notice is that you’ll need to do this per set configuration individually. Sounds like a lot of work but it isn’t. If you have two similar setups you can always copy an existing set and modify the existing one.\n\nLet’s assume you have a new fresh configuration set. In order to fill this set with buff, debuff and cooldown information you’ll want to visit <font color='#0094FF'>your nearest boss dummy</font>. Open the Analyser either through the configuration or the minimap button menu so that the analyser window is displayed."}}},
		{ parent = "Analyzing your abilities", title = "Second step: start using abilities", content = {{ type = "text", text = "When opening the analyser will first identify all buffs which are active on you and your pet and display them in the list of identified items. Now you can buff yourself and attack the boss dummy with the abilities causing a debuff or whose cooldown you want to track. You’ll see that the list in the analyser"},
																										{ type = "image", textureType = "nkRebuff", texturePath = "manual/analyser.png", width = 250, height = 178, align = "lastright"},
																										{ type = "text", text = " window will start to grow with new things it was able to identify.\n\nDepending on the type of ability the entry will have a different colour:\n\nWhite	Buff on yourself\nGrey		Buff on your pet\nRed		Debuff on your target\nBlue		Cooldown"},
																										{ type = "header", text = "<font color='#FF0000'>IMPORTANT!</font>"},
																										{ type = "text", text = "Some abilities cause two buffs or debuffs with exactly the same name meaning there’ll be two entries in the list which look exactly the same. You’ll need to take some extra care with those.\n\nIf the duration of those buffs or debuffs is the same and if they always will appear at the same time after using the ability, it doesn’t really matter which one you choose.\n\nA good example for this is the skill Ace Shot of the rogue’s ranger soul. When activated it will show two debuffs on the target which last equally long.\n\nIf the buffs do not last equally long you’ll have to identify which of the two entries you actually want to track. A good example of this is Prismatic Armor in the mage’s Elementalist tree. If you cast this ability it will put a buff on the player which lasts an hour. While the buff is active there’s a chance for a proc which has the exact same name but only lasts 10 seconds. If you want to track the later you’ll need to choose the entry in the list with the correct duration. In case you’d want to track both, you can choose to do so. I’d advice to choose a different logo or label for one of the two buffs to know which one is which."}}},
		{ parent = "Analyzing your abilities", title = "Last step: Move entries to your set", content = {{ type = "text", text = "Now when your list is filled with all the buffs, debuffs and cooldowns you want to track you have to tell the analyser which ones you really want to track. The list will most likely contain things like tracking buffs. In order to just care about important things once you have done the analyser we’ll only ‘import’ those you want.\n\nBy left clicking an entry in the list the icon in the last column of that entry will turn <font color='#00FF00'>green</font>. This indicates that this entry will be taken over into the set configuration. You can change that by again clicking on the line and the icon will turn back <font color='#FF0000'>red</font> (meaning it will be ignored).\n\nFinally the only thing left to do is to hit the <font color='#0094FF'>Store this</font> button. This will then move the selected entries into your active configuration set."}}},
		{ parent = "Analyzing your abilities", title = "Modifying an existing set", content = {{ type = "text", text = "The Analyzing process is not a one-time thing. You can always choose to re-Analyze later. In order to modify an existing set you’ll first have to activate the set (See chapter <u>Activating a set</u>) you want to add things to. Then you’ll open up the analyser and proceed just like described above. When clicking the <font color='#0094FF'>Store this</font> button the new selected items will be added to your existing set.\n\nAn important thing to note is that the analyser will <font color='#FF0000'>ignore buffs which you have already transferred</font> to your active configuration set. These buffs will no longer show in the analyser window."}}},
		{ parent = nil, title = "Building your set in 9 easy steps", content = {{ type = "text", text = "To sum things up here are 9 steps to build a new set from scratch using the default icon settings:\n\n1. Create a new set using the create or copy button in the settings pane\n\n2. Edit the name of the set\n\n3. Open Analyser and start using your abilities\n\n4. Select the abilities you want to track by left-clicking them (square turns <font color='#00FF00'>green</font>) and then move them to the set configuration by clicking on the <font color='#FF0000'>Store this</font> button\n\n5. Go to the Abilities tab and select the ability type\n\n6. Activate each ability in the list you want to track by right-clicking (square turns <font color='#00FF00'>green</font>)\n\n7. Create an icon for each activated ability by left-clicking the ability and then click on the create button in the icon definition area\n\n8. Move the icon to the desired on-screen position\n\n9. Repeat steps 5-8 for each of the abilities types you want to track\n"}}},
		{ parent = nil, title = "Reversed buff and debuff mode", content = {{ type = "text", text = "This feature of nkRebuff is mainly targeting at healers. I'm not sure if other play styles would also benefit from it, so play around with it and see if it's useful for you as well!\n\nThis mode has to be activated per ability (e.g. icon). So you'll have to go the icon edit pane of an ability and uncheck the option <font color='#0094FF'>use defaults</font>. Then you'll have to check the option <font color='#0094FF'>Show if active</font>.\n\nNormally nkRebuff shows you icons to warn you that the buff or debuff is about to run out. This mode <font color='#FF0000'>reverses</font> that behaviour. Instead nkRebuff will show you the buff or debuff <font color='#FF0000'>as long as it's active</font>"}}},
		{ parent = nil, title = "Miscellaneous"},
		{ parent = "Miscellaneous", title = "Moving the minimap button", content = {{ type = "text", text = "In order to move the minimap button you just have to right-click and hold the mouse button. Then you can drag it to where you want to place it on your ui. This position will immediately by stored.\n\n<font color='#FF0000'>You cannot move the minimap button while in combat!</font>"}}},
		{ parent = "Miscellaneous", title = "Easy rebuffing with the Rebuff button", content = {{ type = "text", text = "As nkRebuff always knows when a configured buff has run out it was quite easy to implement a function which helps you in rebuffing. Instead of having to put all the various buffs on your action bars you can use the ReRebuff button or bar."},
																								{ type = "image", textureType = "nkRebuff", texturePath = "manual/reRebuffButton.png", width = 46, height = 45, align = "lastright"},
																								{ type = "header", text = "Using the bar layout"},
																								{ type = "text", text = "Depending on your configuration settings a button or a bar will be shown. The bar is always shown and will show all buffs configured for the current set. If something needs to be rebuff the icon will be clearly visible. Otherwise it's visibility is reduced. Simply click on a button to rebuff the corresponding buff."},
																								{ type = "header", text = "Using the single button"},
																								{ type = "text", text = "If you choose to use a single button that will always be displayed or only if something needs to be rebuffed.\n\nAs soon as a buff is missing the button will turn <font color='#FF0000'>red</font>. You can then click the button and it will apply a single buff. If more than one buff is missing you have to click the button until all buffs are active again. The red tint will vanish (if the button is always shown) or the button itself will vanish (if the button is only shown if rebuff is needed)."},
																								{ type = "header", text = "Moving the button / bar"},
																								{ type = "text", text = "You can drag & drop the button to wherever you want it to be shown on the screen. The configuration has to be open for the button to be moveable. Therefore while the configuration is open the button will always be shown no matter what the setting to display the button is.\n\n<font color='#FF0000'>The button cannot be moved while you’re in combat!</font>\n\nAlso due to Rift’s API restrictions it’s not possible to update the button while in combat. So if a buff runs out mid-combat you’ll have to wait until you leave combat for the button to update itself."}}},
		{ parent = "Miscellaneous", title = "Importing / Exporting sets", content = {	{ type = "text", text = "It is possible to import and export sets so that you can share them with other players. This is obviously very helpful for supporting standard builds among the community.\n\n"},
																						{ type = "image", textureType = "nkRebuff", texturePath = "manual/importExport.png", width = 250, height = 120, align = "lastright"},
																						{ type = "text", text = "You'll find the exporting and important functionality in the tab import/export of the configuration."},
																						{ type = "header", text = "Exporting a set"},
																						{ type = "text", text = "The export function will always export the <font color='#0094FF'>currently active set</font>. If you want to export a different set than the currently active one, please change the set by either of the available methods (first tab in config, command line or minimap button menu). As soon as you're ready to export the set, simply click on the button <font color='#0094FF'>export</font> and the big field in the middle will fill with rather cryptic text. Don't worry this is normal. The configuration is compressed to make it shorter and easier to share.\n\nThe complete set will be exported including all information about the abilities, their status and icons. What will not be overwritten is of course the default icon configuration.\n\nIf your set relies on a specific default configuration it might be useful to set individual options per icon. Otherwise the set might not look well on another player's screen if he uses different default settings.\n\nAs soon as you have exported the set you can click into the big field and hit the key combinations CTRL+A and CTRL+C. This will copy the data into the clipboard. From there you can then share it by mail or message boards."},
																						{ type = "header", text = "Importing a set"},
																						{ type = "text", text = "Importing is even simpler than exporting. All you have to do is get an previously exported configuration and paste that into the big field on the import/export tab. Then you just have to click on the import button and the set will be imported.\n\nThe addon will check if an existing set has the same name as the one being imported. If so a confirmation dialog will be shown, asking you if the existing set is to be overwritten. If you confirm this, then the existing set will be replaced. If you decline the set will be imported under the name 'Copy of ...'.\n\nIf the set cannot be imported due to truncated or incorrect data you'll receive an error message.\n\n"},
																						 }},
		{ parent = "Miscellaneous", title = "Command line options", content = {	{ type = "text", text = "nkRebuff can be called through the command line by either of the following commands:\n\n/nkrb\n/nkr\n/nkRebuff\n\nEither of theses will open the configuration window if no other options are supplied"},
																				{ type = "header", text = "Resetting the configuration"},
																				{ type = "text", text = "Using the command <font color='#0094FF'>/nkrb reset</font> you can completely reset the configuration. Normally you shouldn't really need this. However if needed the command is there. Just note that <font color='#FF0000'>the configuration is lost and cannot be recovered!</font>"},
																				{ type = "header", text = "Activating a set"},
																				{ type = "text", text = "It is possible to activate a nkRebuff set using the command line. Use the command <font color='#0094FF'>/nkrb set=X</font> where X is the number of the set.\n\nYou can find out the set number in the configuration. To the top right the number and set name of the active set are displayed. By changing the set through the configuration's selection box you can find out the number of each set quite easily.\n\n"},																				
																				}},
}


