local addonInfo, privateVars = ...

privateVars.buffs = {}
privateVars.buffs.eventsInit = false
privateVars.buffs.units = {}
privateVars.buffs.level = 0

privateVars.buffs.activeBuffs = {}
privateVars.buffs.activeCooldowns = {}

---------- BUFF EVENT FUNCTIONS ----------

function nkRebuff.buffAdd (_, unit, buffs)
	
	if nkRebuff.IsValidUnit (unit) == false then return end
	
	if privateVars.buffs.activeBuffs[unit] == nil then privateVars.buffs.activeBuffs[unit] = {} end
	
	for id, buffType in pairs(buffs) do
		if buffType == true then		
			local buffDetails = Inspect.Buff.Detail(unit, id)						
			if buffDetails ~= nil then
				local thisBuffKey = buffDetails.type or buffDetails.name
				
				--nkRebuff.logDebug ("new buff on unit " .. unit .. ": " .. buffDetails.name .. "(".. id .. ") buffType: " .. thisBuffKey)
				
				nkRebuff.processBuffAdd(unit, id, thisBuffKey)
			end
		else
			nkRebuff.processBuffAdd(unit, id, buffType)
		end
	end

end

function nkRebuff.buffRemove(_, unit, buffs)

	for id, _ in pairs(buffs) do
		if privateVars.buffs.activeBuffs[unit] ~= nil then
			if privateVars.buffs.activeBuffs[unit][id] ~= nil then
				--nkRebuff.logDebug ("removing buff on unit " .. unit .. ": " .. id)
			
				nkRebuff.processBuffRemove(unit, id)				
			end
		end
	end

end


function nkRebuff.buffChange (_, unit, buffs)
	-- maybe needed for stack changes
end
---------- COOLDOWN EVENT FUNCTIONS ----------

function nkRebuff.cooldownBegin (_, cooldowns)

	for k, v in pairs (cooldowns) do
		if v > 1.5 then
	
			if privateVars.cooldownCache[k] == nil then	
				privateVars.cooldownCache[k] = Inspect.Ability.New.Detail(k)
			end

			if privateVars.cooldownCache[k] ~= nil then -- only process if was able to get ability details
				privateVars.cooldownCache[k].currentCooldownBegin = Inspect.Time.Frame()
				--print (k .. ' - begin: ' .. Inspect.Time.Frame())
			
--				if privateVars.cooldownCache[k] ~= nil then			
					privateVars.buffs.activeCooldowns[k] = privateVars.cooldownCache[k]
					nkRebuff.ProcessCooldownBegin(k)
--				end				
			end
		end
	end

end

function nkRebuff.cooldownEnd(_, cooldowns)

	for k, v in pairs (cooldowns) do
		if privateVars.buffs.activeCooldowns[k] ~= nil then
			privateVars.buffs.activeCooldowns[k] = nil
			--nkRebuff.ProcessCooldownBegin(k)
			nkRebuff.ProcessCooldownEnd(k)
		end
	end

end

---------- UNIT EVENT FUNCTIONS ----------

function nkRebuff.UnitAvailable(_, unit)

	for k, v in pairs (unit) do
		if v == 'player' then
			local playerDetails = nkRebuff.getUnitDetails (k)
			
			--nkRebuff.logDebug ("player available")
						
			privateVars.buffs.units.player = k
			
			if privateVars.buffs.eventsInit == false then
				nkRebuff.InitEvents()
				nkRebuff.LoadCurrentBuffs(k, true)
				nkRebuff.CheckMissingBuffs()
				nkRebuff.CheckCooldownsReady()
				
				nkRebuff.setReBuffMacro()
			end
		elseif v == 'player.pet' then
			privateVars.buffs.units.playerPet = k
			nkRebuff.getUnitDetails (privateVars.buffs.units.playerPet)
			nkRebuff.CheckMissingBuffsPet()
		end
	end

end

function nkRebuff.PlayerTargetChange (_, unit)
	
	if privateVars.buffs.units.playerTarget ~= nil and privateVars.buffs.activeBuffs[privateVars.buffs.units.playerTarget] ~= nil then
		privateVars.buffs.activeBuffs[privateVars.buffs.units.playerTarget] = nil
	end

	for k, v in pairs (privateVars.uiElements.icons.debuffsTarget) do
		v:SetVisible(false)
	end
	
	if unit ~= false then
		local unitDetails = Inspect.Unit.Detail(unit)
		if unitDetails ~= nil then
			if unitDetails.relation ~= 'friendly' then	
				privateVars.buffs.units.playerTarget = unit
				privateVars.buffs.activeBuffs[privateVars.buffs.units.playerTarget] = {}
				nkRebuff.LoadCurrentBuffs(unit, false)
				nkRebuff.CheckMissingDebuffs()
			else
				privateVars.buffs.units.playerTarget = nil
			end
		end
	end
	
end

function nkRebuff.InitEvents()

	--nkRebuff.logDebug ("init events")

	--Command.Event.Attach(Event.Unit.Add, nkRebuff.UnitAdd, "nkRebuff.Unit.Add")
	--Command.Event.Attach(Event.LibUnitChange.player.target.Change, nkRebuff.PlayerTargetChange, "nkRebuff.Player.Target.Change")
	Command.Event.Attach(Library.LibUnitChange.Register("player.target"), nkRebuff.PlayerTargetChange, "nkRebuff.Player.Target.Change")
		 
	Command.Event.Attach(Event.Buff.Add, nkRebuff.buffAdd, "nkRebuff.Buff.Add")
	Command.Event.Attach(Event.Buff.Remove, nkRebuff.buffRemove, "nkRebuff.Buff.Remove")
	Command.Event.Attach(Event.Buff.Change, nkRebuff.buffChange, "nkRebuff.Buff.Change")
	
	Command.Event.Attach(Event.System.Update.Begin, nkRebuff.systemUpdate, "nkRebuff.System.Update.Begin")
		
	Command.Event.Attach(Event.Ability.New.Cooldown.Begin, nkRebuff.cooldownBegin, "nkRebuff.Cooldown.Begin")
	Command.Event.Attach(Event.Ability.New.Cooldown.End, nkRebuff.cooldownEnd, "nkRebuff.Cooldown.End")
				
	privateVars.buffs.eventsInit = true

end

Command.Event.Attach(Event.Unit.Availability.Full, nkRebuff.UnitAvailable, "nkRebuff.Unit.Availability.Full")

---------- HELPER FUNCTIONS ----------

function nkRebuff.LoadCurrentBuffs (unit, loadCooldowns)

	--nkRebuff.logDebug ("load current buff on unit: " .. unit)

	if Inspect.System.Secure() == false then Command.System.Watchdog.Quiet() end -- get some more time before performance warning comes up	

	local bufflist = Inspect.Buff.List(unit)
	if bufflist == nil then return end
	local buffDetails = Inspect.Buff.Detail(unit, bufflist)
		
	--if privateVars.buffs.activeBuffs[unit] == nil then privateVars.buffs.activeBuffs[unit] = {} end
	privateVars.buffs.activeBuffs[unit] = {}
	
	if buffDetails ~= nil then	
		for k, v in pairs (buffDetails) do
			privateVars.buffCache[k] = v
		
			local buffKey = v.type or v.name
			
			if buffDetails.curse == true or buffDetails.debuff == true then
				if buffDetails.caster == privateVars.buffs.units.player then -- ignore debuffs casted by other players
					--if buffKey ~= nil then privateVars.buffs.activeBuffs[unit][k] = v end
					--print ('debuff ' .. privateVars.buffCache[k].name .. ' new begin: ' .. v.begin)
					privateVars.buffs.activeBuffs[unit][k] = { key = buffKey, begin = v.begin }
				end
			else						
				--if buffKey ~= nil then privateVars.buffs.activeBuffs[unit][k] = v end
				privateVars.buffs.activeBuffs[unit][k] = { key = buffKey, begin = v.begin }
			end
		end
	end
	
	if loadCooldowns == true then
		local abilityList = Inspect.Ability.New.List()
		
		for k, v in pairs(abilityList) do
			local abilityDetails = Inspect.Ability.New.Detail(k)
			privateVars.cooldownCache[k] = Inspect.Ability.New.Detail(k)
			
			if abilityDetails ~= nil and abilityDetails.currentCooldownBegin ~= nil then
				if abilityDetails.currentCooldownDuration > 1.5 then
					privateVars.buffs.activeCooldowns[k] = abilityDetails
				end
			end
		end
	end

end

function nkRebuff.IsValidUnit(unit)

	if privateVars.buffs.units == nil then return false end

	if unit == privateVars.buffs.units.player or unit == privateVars.buffs.units.playerPet or unit == privateVars.buffs.units.playerTarget then
		return true
	else
		return false
	end

end

---------- QUERY FUNCTIONS ----------

function nkRebuff.IsBuffActive(unit, buffKey)

	local buffDetails = nkRebuff.GetStoredBuffData (unit)		
	if buffDetails == nil then return false end
	
	for k, v in pairs(buffDetails) do
		--local thisBuffKey = v.type or v.name
		if v.key == buffKey then return true end
	end
	
	return false	

end

function nkRebuff.IsDebuffActive(unit, debuffKey)

	local debuffDetails = nkRebuff.GetStoredBuffData (unit)
	if debuffDetails == nil then return false end

	for k, v in pairs(debuffDetails) do
		--local thisDebuffKey = v.type or v.name
		if v.key == debuffKey then
			return true 
		end
	end
	
	return false	

end

function nkRebuff.IsCooldownActive(abilityID)

	if privateVars.buffs.activeCooldowns == nil then return false end
	
	for k, v in pairs(privateVars.buffs.activeCooldowns) do
		if k == abilityID then return true end
	end
	
	return false

end

function nkRebuff.GetStoredBuffData (unit)

	local buffDetails

	if unit == 'player' then
		buffDetails = privateVars.buffs.activeBuffs[privateVars.buffs.units.player]
	elseif unit == 'player.pet' then
		buffDetails = privateVars.buffs.activeBuffs[privateVars.buffs.units.playerPet]
	elseif unit == 'player.target' then
		buffDetails = privateVars.buffs.activeBuffs[privateVars.buffs.units.playerTarget]		
	elseif nkRebuff.IsValidUnit(unit) then
		buffDetails = privateVars.buffs.activeBuffs[unit]
	end
	
	return buffDetails

end

function nkRebuff.GetSystemBuffID (unit, buffKey)

	local buffDetails = nil
	
	if unit == 'player' then
		buffDetails = privateVars.buffs.activeBuffs[privateVars.buffs.units.player]
	elseif unit == 'player.pet' then
		buffDetails = privateVars.buffs.activeBuffs[privateVars.buffs.units.playerPet]
	elseif unit == 'player.target' then
		buffDetails = privateVars.buffs.activeBuffs[privateVars.buffs.units.playerTarget]		
	elseif nkRebuff.IsValidUnit(unit) then
		buffDetails = privateVars.buffs.activeBuffs[unit]
	end
	
	if buffDetails == nil then return nil end
	
	for k, v in pairs(buffDetails) do
		--local thisBuffKey = v.type or v.name
		if v.key == buffKey then return k end
	end
	
	return nil

end

function nkRebuff.GetCooldownDetails (abilityID)

	return privateVars.buffs.activeCooldowns[abilityID]

end