local addonInfo, privateVars = ...

privateVars.buffCache = {}
privateVars.cooldownCache = {}

function nkRebuff.processBuffAdd(unit, systemID, buffType)

	local didBuffDetail = false

	if privateVars.buffCache[systemID] == nil then
		--print ('getting buff: ' .. unit .. ' ' .. systemID)
		privateVars.buffCache[systemID] = Inspect.Buff.Detail(unit, systemID)
		didBuffDetail = true
	end
	
	local thisBuffKey = privateVars.buffCache[systemID].type or privateVars.buffCache[systemID].name
	
	--nkRebuff.logDebug ("process buff add: " .. unit .. "(" .. systemID .. ") " .. privateVars.buffCache[systemID].name .. " buffType: " .. thisBuffKey)
	
	if unit == privateVars.buffs.units.player or unit == privateVars.buffs.units.playerPet then
		
		local index, unitType, petFlag = 'buffsPlayer', 'player', false
		if unit == privateVars.buffs.units.playerPet then index, unitType, petFlag = 'buffsPet', 'player.pet', true end
	
		privateVars.buffs.activeBuffs[unit][systemID] = { key = buffType, begin = Inspect.Time.Frame() }
	
		if privateVars.buffCache[systemID].curse == true or privateVars.buffCache[systemID].debuff == true then return end -- later

		if nkRBSets[nkRBConfig.activeSet][index] == nil then return end -- not tracked
		if nkRBSets[nkRBConfig.activeSet][index][thisBuffKey] == nil then -- not tracked
			nkRebuff.CheckForNewID('buffsPlayer', privateVars.buffCache[systemID])
			if nkRBSets[nkRBConfig.activeSet][index][thisBuffKey] == nil then return end
		end 
		
		if nkRBSets[nkRBConfig.activeSet][index][thisBuffKey].active == false then return end
		if nkRBIcons[nkRBConfig.activeSet][index] == nil or nkRBIcons[nkRBConfig.activeSet][index][thisBuffKey] == nil then return end -- tracking not active or no icon created
		
		if nkRBIcons[nkRBConfig.activeSet][index][thisBuffKey].showIfActive == true then
			local icon = nkRebuff.displayOrCreateIcon (index, thisBuffKey, privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)
			icon:SetTimer("")
			if privateVars.buffCache[systemID].stack ~= nil then icon:SetStack(privateVars.buffCache[systemID].stack) else icon:SetStack("") end			
		else
			if privateVars.uiElements.icons[index][thisBuffKey] ~= nil then 
				privateVars.uiElements.icons[index][thisBuffKey]:SetVisible(false)
				privateVars.uiElements.icons[index][thisBuffKey]:SetInfoBlock(privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)
			end
			nkRebuff.setReBuffMacro(thisBuffKey)
		end

	elseif unit == privateVars.buffs.units.playerTarget then
		-- scan for debuffs on target
		
		if privateVars.buffCache[systemID].curse == false and privateVars.buffCache[systemID].debuff == false then return end
		if privateVars.buffCache[systemID].caster ~= privateVars.buffs.units.player then return end -- ignore debuffs casted by other players
				
		privateVars.buffs.activeBuffs[unit][systemID] = { key = buffType, begin = Inspect.Time.Frame() }
		
		if nkRBSets[nkRBConfig.activeSet].debuffsTarget == nil then return end 
		if nkRBSets[nkRBConfig.activeSet].debuffsTarget[thisBuffKey] == nil then
			nkRebuff.CheckForNewID('debuffsTarget', privateVars.buffCache[systemID])
			if nkRBSets[nkRBConfig.activeSet].debuffsTarget[thisBuffKey] == nil then return end
		end
			
		if nkRBSets[nkRBConfig.activeSet].debuffsTarget[thisBuffKey].active == false then return end -- not tracked
		if nkRBIcons[nkRBConfig.activeSet].debuffsTarget == nil or nkRBIcons[nkRBConfig.activeSet].debuffsTarget[thisBuffKey] == nil then return end -- no icon created
		
		if nkRBIcons[nkRBConfig.activeSet].debuffsTarget[thisBuffKey].showIfActive == true then
			local icon = nkRebuff.displayOrCreateIcon ('debuffsTarget', thisBuffKey, privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)
			icon:SetTimer("")
			if privateVars.buffCache[systemID].stack ~= nil then icon:SetStack(privateVars.buffCache[systemID].stack) else icon:SetStack("") end
		else
			if privateVars.uiElements.icons.debuffsTarget[thisBuffKey] ~= nil then
		
				if didBuffDetail == false and nkRBIcons[nkRBConfig.activeSet].debuffsTarget[thisBuffKey].showStack == true then
					--print ('getting debuff: ' .. unit .. ' ' .. systemID)
					privateVars.buffCache[systemID] = Inspect.Buff.Detail(unit, systemID)				
					didBuffDetail = true
				end
			
				privateVars.uiElements.icons.debuffsTarget[thisBuffKey]:SetInfoBlock(privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)
				privateVars.uiElements.icons.debuffsTarget[thisBuffKey]:SetVisible(false) 
			end
		end
	end

end

function nkRebuff.processBuffRemove(unit, systemID)

	local buffDetails = privateVars.buffCache[systemID]
	if buffDetails == nil then return end
	
	local thisBuffKey = buffDetails.type or buffDetails.name
	
	--nkRebuff.logDebug ("process buff remove: " .. unit .. "(" .. systemID .. ") " .. buffDetails.name .. " buffType: " .. thisBuffKey)
	
	if unit == privateVars.buffs.units.player or unit == privateVars.buffs.units.playerPet then
		if buffDetails.curse == true or buffDetails.debuff == true then return end -- later
		privateVars.buffs.activeBuffs[unit][systemID] = nil
		
		local index = 'buffsPlayer'
		if unit == privateVars.buffs.units.playerPet then index = 'buffsPet' end
		
		-- scann for buffs on player or pet
		
		if nkRBSets[nkRBConfig.activeSet][index] == nil then return end -- not tracked
		if nkRBSets[nkRBConfig.activeSet][index][thisBuffKey] == nil then return end -- not tracked
		if nkRBSets[nkRBConfig.activeSet][index][thisBuffKey].active ~= true then return end -- tracking not active
		if nkRBIcons[nkRBConfig.activeSet][index][thisBuffKey] == nil then return end -- no icon created
		
		nkRebuff.setReBuffMacro(thisBuffKey) -- this check needs to be here cause otherwise it will be called even if a buff is removed which is not event tracked
		
		if nkRBConfig.combatOnly == true and privateVars.isSecure == false then return end
				
		if nkRBIcons[nkRBConfig.activeSet][index][thisBuffKey].showIfActive == true then
			if privateVars.uiElements.icons[index][thisBuffKey] ~= nil then
				privateVars.uiElements.icons[index][thisBuffKey]:SetInfoBlock(privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description) 
				privateVars.uiElements.icons[index][thisBuffKey]:SetVisible(false) 
			end
		else
			local icon = nkRebuff.displayOrCreateIcon (index, thisBuffKey, privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)
			icon:SetTimer("")
			if privateVars.buffCache[systemID].stack ~= nil then icon:SetStack(privateVars.buffCache[systemID].stack) else icon:SetStack("") end
		end	
	else
		-- scan for debuffs on target
		
		if buffDetails.curse == false and buffDetails.debuff == false then return end
		
		if buffDetails.caster ~= privateVars.buffs.units.player then return end -- ignore debuffs casted by other players
		
		privateVars.buffs.activeBuffs[unit][systemID] = nil
		
		if nkRBConfig.combatOnly == true and privateVars.isSecure == false then return end
		
		if nkRBSets[nkRBConfig.activeSet].debuffsTarget == nil then return end -- not tracked
		if nkRBSets[nkRBConfig.activeSet].debuffsTarget[thisBuffKey] == nil then return end -- not tracked
		if nkRBSets[nkRBConfig.activeSet].debuffsTarget[thisBuffKey].active ~= true then return end -- tracking not active
		
		if nkRBIcons[nkRBConfig.activeSet] == nil or nkRBIcons[nkRBConfig.activeSet].debuffsTarget == nil or nkRBIcons[nkRBConfig.activeSet].debuffsTarget[thisBuffKey] == nil then return end -- no icon configured
		
		if nkRBIcons[nkRBConfig.activeSet].debuffsTarget[thisBuffKey].showIfActive == true then
			if privateVars.uiElements.icons.debuffsTarget ~= nil and privateVars.uiElements.icons.debuffsTarget[thisBuffKey] ~= nil then 
				privateVars.uiElements.icons.debuffsTarget[thisBuffKey]:SetInfoBlock(privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)	
				privateVars.uiElements.icons.debuffsTarget[thisBuffKey]:SetVisible(false) 
			end
		else			
			local icon = nkRebuff.displayOrCreateIcon ('debuffsTarget', thisBuffKey, privateVars.buffCache[systemID].name, privateVars.buffCache[systemID].description)		
			icon:SetTimer("")
			if privateVars.buffCache[systemID].stack ~= nil then icon:SetStack(privateVars.buffCache[systemID].stack) else icon:SetStack("") end
		end
	end

end

function nkRebuff.ProcessCooldownBegin (key)

	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer == nil then return end -- not tracked
	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer[key] == nil then return end -- not tracked
	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer[key].active == false then return end -- tracking not active
	
	if privateVars.uiElements.icons.cooldownsPlayer[key] ~= nil then privateVars.uiElements.icons.cooldownsPlayer[key]:SetVisible(false) end

end

function nkRebuff.ProcessCooldownEnd (key)

	local iconOnCD = nkRebuff.getIconCDState (key)
	
	for k, v in pairs (iconOnCD) do
		privateVars.uiElements.icons[v.iconType][v.id]:SetAlpha(1)
		nkRebuff.storeIconCDState (v.iconType, v.id, nil)
	end
	
	if nkRBConfig.combatOnly == true and privateVars.isSecure == false then return end

	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer == nil then return end -- not tracked
	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer[key] == nil then return end -- not tracked
	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer[key].active == false then return end -- tracking not active
	
	--print ('cooldown end')
	local icon = nkRebuff.displayOrCreateIcon ('cooldownsPlayer', key, privateVars.cooldownCache[key].name, privateVars.cooldownCache[key].description)		
	--icon:SetTimer("")
	--icon:SetStack("")

end

function nkRebuff.CheckMissingBuffs(overrideCombatCheck)

	--nkRebuff.logDebug ("check missing buffs on player")

	if nkRBSets[nkRBConfig.activeSet].buffsPlayer == nil then
		 --nkRebuff.logDebug ("no buffs configured")
		return 
	end -- not tracked
	
	if overrideCombatCheck ~= true and nkRBConfig.combatOnly == true and privateVars.isSecure == false then return end
	
	for k, v in pairs (nkRBSets[nkRBConfig.activeSet].buffsPlayer) do	
		if v.active == true then
			if nkRebuff.CheckForMissingConfigData (privateVars.buffs.units.player, 'buffsPlayer', k) == true then
				if nkRBIcons[nkRBConfig.activeSet].buffsPlayer[k] ~= nil then												
					if nkRebuff.IsBuffActive(privateVars.buffs.units.player, k) == nkRBIcons[nkRBConfig.activeSet].buffsPlayer[k].showIfActive then
						nkRebuff.displayOrCreateIcon ('buffsPlayer', k )
					end
				end				
			end
		end
	end

end

function nkRebuff.CheckMissingBuffsPet(overrideCombatCheck)

	--print("CheckMissingBuffsPet")

	if nkRBSets[nkRBConfig.activeSet].buffsPet == nil then
		--print("not checked") 
		return		
	end -- not tracked
	
	if overrideCombatCheck ~= true and nkRBConfig.combatOnly == true and privateVars.isSecure == false then
		--print("overrideCombatCheck: ", overrideCombatCheck)
		--print("combatOnly: ", nkRBConfig.combatOnly)
		--print("isSecure: ", privateVars.isSecure) 
		return 
	end
	
	for k, v in pairs (nkRBSets[nkRBConfig.activeSet].buffsPet) do
		if v.active == true then
			if nkRebuff.CheckForMissingConfigData (privateVars.buffs.units.playerPet, 'buffsPet', k) == true then
				if nkRBIcons[nkRBConfig.activeSet].buffsPet[k] ~= nil then
					--print (nkRebuff.IsBuffActive(privateVars.buffs.units.playerPet, k), nkRBIcons[nkRBConfig.activeSet].buffsPet[k].showIfActive)
					if nkRebuff.IsBuffActive(privateVars.buffs.units.playerPet, k) == nkRBIcons[nkRBConfig.activeSet].buffsPet[k].showIfActive then
						nkRebuff.displayOrCreateIcon ('buffsPet', k )
					end
				else
					--print ("nkRBIcons ist nil für " .. k)				
				end
			end
		end
	end

end

function nkRebuff.CheckMissingDebuffs(overrideCombatCheck)

	if nkRBSets[nkRBConfig.activeSet].debuffsTarget == nil then return end -- not tracked
	
	--if overrideCombatCheck ~= true and nkRBConfig.combatOnly == true and privateVars.isSecure == false then return end
	
	for k, v in pairs (nkRBSets[nkRBConfig.activeSet].debuffsTarget) do
		if v.active == true then
			if nkRebuff.CheckForMissingConfigData (privateVars.buffs.units.playerTarget, 'debuffsTarget', k) == true then
				if nkRBIcons[nkRBConfig.activeSet].debuffsTarget[k] ~= nil then
					if nkRebuff.IsDebuffActive(privateVars.buffs.units.playerTarget, k) == nkRBIcons[nkRBConfig.activeSet].debuffsTarget[k].showIfActive then
						nkRebuff.displayOrCreateIcon ('debuffsTarget', k )

						if overrideCombatCheck == true or nkRBConfig.combatOnly == false or privateVars.isSecure == true then
							privateVars.uiElements.icons.debuffsTarget[k]:SetVisible(true)
						else
							privateVars.uiElements.icons.debuffsTarget[k]:SetVisible(false)
						end
					end
				end
			end
		end
	end

end

function nkRebuff.CheckCooldownsReady()

	if nkRBSets[nkRBConfig.activeSet].cooldownsPlayer == nil then return end -- not tracked
	
	if nkRBConfig.combatOnly == true and privateVars.isSecure == false then return end
	
	for k, v in pairs (nkRBSets[nkRBConfig.activeSet].cooldownsPlayer) do
		if v.active == true then
			if nkRebuff.IsCooldownActive(k) == false then				
				if privateVars.uiElements.icons.cooldownsPlayer[k] == nil then
					if nkRBIcons[nkRBConfig.activeSet].cooldownsPlayer[k] ~= nil then
						--print ('cooldowns ready')			
						nkRebuff.displayOrCreateIcon ('cooldownsPlayer', k )
					end
				end
			end
		end
	end

end

function nkRebuff.CheckForMissingConfigData (unitType, buffType, buffID)

	if nkRBData[nkRBConfig.activeSet][buffType][buffID] == nil then
		local buffDetails = nkRebuff.GetStoredBuffData (unitType, buffType)
		if buffDetails ~= nil and buffDetails[buffID] ~= nil then 
			nkRebuff.printError(privateVars.langTexts.errTypeConfig, string.format(privateVars.langTexts.errMissingBuffData, buffDetails[buffID].name))
		else
			nkRebuff.printError(privateVars.langTexts.errTypeConfig, string.format(privateVars.langTexts.errMissingBuffData, buffID))
		end
		nkRBSets[nkRBConfig.activeSet][buffType][buffID] = nil
		return false
	else
		return true
	end

end

function nkRebuff.CheckForNewID(buffType, buffDetails, isPet)

	if nkRBConfig.checkforupdates == false then return end

	local data, icons, set = nkRBData[nkRBConfig.activeSet][buffType], nkRBIcons[nkRBConfig.activeSet][buffType], nkRBSets[nkRBConfig.activeSet][buffType]
	
	for k, v in pairs (data) do
		if v.name == buffDetails.name then
			local update = false
		
--			print (buffDetails.abilityNew, v.ability)
--			print (buffDetails.type, k)			
--			print (EnKai.tools.math.round(buffDetails.duration, 0), EnKai.tools.math.round(v.duration, 0))
		
			if buffDetails.abilityNew ~= v.ability then
				-- found new version of configured thing
				update = true	
			elseif buffDetails.type ~= nil and buffDetails.type ~= k then
				-- found new version of configured thing
				if EnKai.tools.math.round(buffDetails.duration, 0) == EnKai.tools.math.round(v.duration, 0) then
					update = true
				end
			end
			
			if update == true then
			
				Command.Console.Display("general", true, string.format(privateVars.langTexts.msgNewItemVersion, buffDetails.name), true)
			
				local newBuffKey = buffDetails.type or buffDetails.name
				local oldBuffKey = k or v.name
				
				if newBuffKey == oldBuffKey then
					data[oldBuffKey].ability = buffDetails.abilityNew
					data[oldBuffKey].icon = buffDetails.icon
					data[oldBuffKey].duration = buffDetails.duration
				else
					data[newBuffKey] = EnKai.tools.table.copy(data[k])
					data[newBuffKey].ability = buffDetails.abilityNew
					data[newBuffKey].icon = buffDetails.icon
					data[newBuffKey].duration = buffDetails.duration
					
					data[oldBuffKey] = nil
					
					if icons[oldBuffKey] ~= nil then
						icons[newBuffKey] = EnKai.tools.table.copy(icons[k])
						icons[oldBuffKey] = nil
					end
					
					if set[oldBuffKey] ~= nil then
						set[newBuffKey] = EnKai.tools.table.copy(set[k])
						set[oldBuffKey] = nil
					end			
				end
			end
		end
	end

end